Package: pbdZMQ
Version: 0.3-14
Date: 2025-04-12
Title: Programming with Big Data -- Interface to 'ZeroMQ'
Authors@R: c(person("Wei-Chen", "Chen", role = c("aut", "cre"), email =
        "wccsnow@gmail.com"),
        person("Drew", "Schmidt", role = "aut"),
        person("Christian", "Heckendorf", role = "aut", comment = "file transfer"),
        person("George", "Ostrouchov", role = "aut", comment = "Mac OSX"),
        person("Whit", "Armstrong", role = "ctb", comment = "some functions
                are modified from the rzmq package for backwards compatibility"),
        person("Brian", "Ripley", role = "ctb",
               comment = "C code of shellexec, and Solaris"),
        person("R Core team", role = "ctb", comment = "some functions and
                headers are copied or modified from the R source code"),
        person("Philipp", "A.", role = "ctb", comment = "Fedora"),
        person("Elliott Sales", "de Andrade", role = "ctb", comment = "sprintf"),
        person("Spencer", "Aiello", role = "ctb", comment = "windows conf"),
        person("Paul", "Andrey", role = "ctb", comment = "Mac OSX conf"),
        person("Panagiotis", "Cheilaris", role = "ctb", comment = "add serialize version"),
        person("Jeroen", "Ooms", role = "ctb", comment = "clang++ on MacOS ARM64"),
        person("ZeroMQ authors", role = c("aut", "cph"), comment = "source files in 'src/zmq_src/'")
	)
Depends: R (>= 3.5.0)
LazyLoad: yes
Copyright: See files AUTHORS, COPYING, and COPYING.LESSER in
        'inst/zmq_copyright/' for the 'ZeroMQ' source files in
        'src/zmq_src/' which are under GPL-3.
Description: 'ZeroMQ' is a well-known library for high-performance
    asynchronous messaging in scalable, distributed applications.  This
    package provides high level R wrapper functions to easily utilize
    'ZeroMQ'. We mainly focus on interactive client/server programming
    frameworks. For convenience, a minimal 'ZeroMQ' library (4.2.2)
    is shipped with 'pbdZMQ', which can be used if no system installation
    of 'ZeroMQ' is available.  A few wrapper functions compatible with
    'rzmq' are also provided.
SystemRequirements: Linux, Mac OSX, and Windows, or 'ZeroMQ' library >=
        4.0.4. Solaris 10 needs 'ZeroMQ' library 4.0.7 and 'OpenCSW'.
StagedInstall: TRUE
License: GPL-3
URL: https://pbdr.org/
BugReports: https://github.com/snoweye/pbdZMQ/issues
NeedsCompilation: yes
Maintainer: Wei-Chen Chen <wccsnow@gmail.com>
RoxygenNote: 7.2.3
Packaged: 2025-04-13 00:34:47 UTC; snoweye
Author: Wei-Chen Chen [aut, cre],
  Drew Schmidt [aut],
  Christian Heckendorf [aut] (file transfer),
  George Ostrouchov [aut] (Mac OSX),
  Whit Armstrong [ctb] (some functions are modified from the rzmq package
    for backwards compatibility),
  Brian Ripley [ctb] (C code of shellexec, and Solaris),
  R Core team [ctb] (some functions and headers are copied or modified
    from the R source code),
  Philipp A. [ctb] (Fedora),
  Elliott Sales de Andrade [ctb] (sprintf),
  Spencer Aiello [ctb] (windows conf),
  Paul Andrey [ctb] (Mac OSX conf),
  Panagiotis Cheilaris [ctb] (add serialize version),
  Jeroen Ooms [ctb] (clang++ on MacOS ARM64),
  ZeroMQ authors [aut, cph] (source files in 'src/zmq_src/')
Repository: CRAN
Date/Publication: 2025-04-13 05:00:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 20:07:13 UTC; unix

Package: pbdZMQ
Version: 0.3-14
Date: 2025-04-12
Title: Programming with Big Data -- Interface to 'ZeroMQ'
Authors@R: c(person("Wei-Chen", "Chen", role = c("aut", "cre"), email =
        "wccsnow@gmail.com"),
        person("Drew", "Schmidt", role = "aut"),
        person("Christian", "Heckendorf", role = "aut", comment = "file transfer"),
        person("George", "Ostrouchov", role = "aut", comment = "Mac OSX"),
        person("Whit", "Armstrong", role = "ctb", comment = "some functions
                are modified from the rzmq package for backwards compatibility"),
        person("Brian", "Ripley", role = "ctb",
               comment = "C code of shellexec, and Solaris"),
        person("R Core team", role = "ctb", comment = "some functions and
                headers are copied or modified from the R source code"),
        person("Philipp", "A.", role = "ctb", comment = "Fedora"),
        person("Elliott Sales", "de Andrade", role = "ctb", comment = "sprintf"),
        person("Spencer", "Aiello", role = "ctb", comment = "windows conf"),
        person("Paul", "Andrey", role = "ctb", comment = "Mac OSX conf"),
        person("Panagiotis", "Cheilaris", role = "ctb", comment = "add serialize version"),
        person("Jeroen", "Ooms", role = "ctb", comment = "clang++ on MacOS ARM64"),
        person("ZeroMQ authors", role = c("aut", "cph"), comment = "source files in 'src/zmq_src/'")
	)
Depends: R (>= 3.5.0)
LazyLoad: yes
Copyright: See files AUTHORS, COPYING, and COPYING.LESSER in
        'inst/zmq_copyright/' for the 'ZeroMQ' source files in
        'src/zmq_src/' which are under GPL-3.
Description: 'ZeroMQ' is a well-known library for high-performance
    asynchronous messaging in scalable, distributed applications.  This
    package provides high level R wrapper functions to easily utilize
    'ZeroMQ'. We mainly focus on interactive client/server programming
    frameworks. For convenience, a minimal 'ZeroMQ' library (4.2.2)
    is shipped with 'pbdZMQ', which can be used if no system installation
    of 'ZeroMQ' is available.  A few wrapper functions compatible with
    'rzmq' are also provided.
SystemRequirements: Linux, Mac OSX, and Windows, or 'ZeroMQ' library >=
        4.0.4. Solaris 10 needs 'ZeroMQ' library 4.0.7 and 'OpenCSW'.
StagedInstall: TRUE
License: GPL-3
URL: https://pbdr.org/
BugReports: https://github.com/snoweye/pbdZMQ/issues
NeedsCompilation: yes
Maintainer: Wei-Chen Chen <wccsnow@gmail.com>
RoxygenNote: 7.2.3
Packaged: 2025-04-13 00:34:47 UTC; snoweye
Author: Wei-Chen Chen [aut, cre],
  Drew Schmidt [aut],
  Christian Heckendorf [aut] (file transfer),
  George Ostrouchov [aut] (Mac OSX),
  Whit Armstrong [ctb] (some functions are modified from the rzmq package
    for backwards compatibility),
  Brian Ripley [ctb] (C code of shellexec, and Solaris),
  R Core team [ctb] (some functions and headers are copied or modified
    from the R source code),
  Philipp A. [ctb] (Fedora),
  Elliott Sales de Andrade [ctb] (sprintf),
  Spencer Aiello [ctb] (windows conf),
  Paul Andrey [ctb] (Mac OSX conf),
  Panagiotis Cheilaris [ctb] (add serialize version),
  Jeroen Ooms [ctb] (clang++ on MacOS ARM64),
  ZeroMQ authors [aut, cph] (source files in 'src/zmq_src/')
Repository: CRAN
Date/Publication: 2025-04-13 05:00:06 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 19:38:50 UTC; unix

# Generated by roxygen2: do not edit by hand

export(.zmqopt_get)
export(.zmqopt_init)
export(.zmqopt_set)
export(ZMQ.MC)
export(ZMQ.PO)
export(ZMQ.SO)
export(ZMQ.SR)
export(ZMQ.ST)
export(address)
export(bind.socket)
export(connect.socket)
export(get.pbdZMQ.ldflags)
export(get.zmq.cppflags)
export(get.zmq.ldflags)
export(init.context)
export(init.socket)
export(ls)
export(overwrite.shpkg.rpath)
export(pbd_opt)
export(random_open_port)
export(random_port)
export(receive.socket)
export(send.socket)
export(test.load.zmq)
export(zmq.bind)
export(zmq.close)
export(zmq.connect)
export(zmq.ctx.destroy)
export(zmq.ctx.new)
export(zmq.disconnect)
export(zmq.getsockopt)
export(zmq.msg.recv)
export(zmq.msg.send)
export(zmq.poll)
export(zmq.poll.free)
export(zmq.poll.get.revents)
export(zmq.poll.length)
export(zmq.recv)
export(zmq.recv.multipart)
export(zmq.recvdir)
export(zmq.recvfile)
export(zmq.send)
export(zmq.send.multipart)
export(zmq.senddir)
export(zmq.sendfile)
export(zmq.setsockopt)
export(zmq.socket)
export(zmq.strerror)
export(zmq.version)
importFrom(stats,runif)
importFrom(utils,unzip)
importFrom(utils,zip)

# pbdZMQ

* **License:** [![License](https://img.shields.io/badge/license-GPL%20v3-orange.svg?style=flat)](https://www.gnu.org/licenses/gpl-3.0.en.html)
* **Download:** [![Download](https://cranlogs.r-pkg.org/badges/pbdZMQ)](https://cran.r-project.org/package=pbdZMQ)
* **Status:** [![Build Status](https://gitlab.com/snoweye/pbdZMQ/badges/master/pipeline.svg)](https://gitlab.com/snoweye/pbdZMQ/-/commits/master) [![Appveyor Build status](https://ci.appveyor.com/api/projects/status/32r7s2skrgm9ubva?svg=true)](https://ci.appveyor.com/project/snoweye/pbdZMQ)
* **Author:** See section below.


pbdZMQ is an R package providing a simplified interface to ZeroMQ with a focus on client/server programming frameworks.  Notably, pbdZMQ should allow for the use of ZeroMQ on Windows platforms.


## Interfaces

The package contains 2 separate interfaces:

1. One modeled after the ZeroMQ C interface (see help("czmq"))
2. One modeled after the rzmq interface (see help("rzmq"))


## Client/Server Example

The primary focus of pbdZMQ is for building client/server interfaces for R.  An example of this can be found in the pbdCS package, which uses this model to control batch MPI servers interactively.  There are also several illustrative examples in the pbdZMQ package vignette.

The basic idea is that you need a server R process, and a separate client R process.  For demonstration/simplicity, assume they are both running on the same machine.  The server we describe here is very basic.  You can see a more detailed example in the pbdZMQ package vignette.


#### Server

Save the following as, say, `server.r` and run it in batch by running `Rscript server.r` from a terminal.

```r
library(pbdZMQ)
ctxt <- init.context()
socket <- init.socket(ctxt, "ZMQ_REP")
bind.socket(socket, "tcp://*:55555")


cat("Client command:  ")
  msg <- receive.socket(socket)

cat(msg, "\n")
send.socket(socket, "Message received!")
```


#### Client

From an interactive R session (not in batch), enter the following:

```r
library(pbdZMQ)
ctxt <- init.context()
socket <- init.socket(ctxt, "ZMQ_REQ")
connect.socket(socket, "tcp://localhost:55555")

send.socket(socket, "1+1")
receive.socket(socket)
```

If all goes well, your message should be sent from the client to the server, before your server terminates.

For an example of how to do this more persistently, see the pbdZMQ package vignette.


## Installation

pbdZMQ requires

* R version 3.0.0 or higher.
* Linux, Mac OSX, Windows, or FreeBSD.
* libzmq >= 4.0.4.
* Solaris 10 requiring external libzmq 4.0.7 and OpenCSW.

A distribution of libzmq is shipped with pbdZMQ for convenience.  However, if you already have a system installation of ZeroMQ, then it is simple to use that with pbdZMQ.  Full details on installation and troubleshooting can be found in the package vignette, located at `inst/doc/pbdZMQ-guide.pdf` of the pbdZMQ source tree.

The package can be installed from the CRAN via the usual `install.packages("pbdZMQ")`, or via the devtools package:

```r
library(devtools)
install_github("RBigData/pbdZMQ")
```


## Citation

When mentioning the pbdZMQ, please cite:

```
@MISC{pbdZMQ2015,
  author = {Chen, W.-C. and Schmidt, D. and Heckendorf, C. and Ostrouchov, G.},
  title = {pbdZMQ: Programming with Big Data -- Interface to ZeroMQ},
  year = {2015},
  note = {R Package, URL https://cran.r-project.org/package=pbdZMQ}
}
```


## Authors

pbdZMQ is authored and maintained by:

* Wei-Chen Chen
* Drew Schmidt
* Christian Heckendorf
* George Ostrouchov

With additional contributions and authors from:

* Whit Armstrong (some functions are modified from rzmq for backwards compatibility)
* Brian Ripley (C code of shellexec)
* The R Core team (some functions are modified from the R source code)
* Philipp A. (Fedora)
* Elliott Sales de Andrade (sprintf version underflow)
* Spencer Aiello (windows conf spacing)
* Paul Andrey (Mac OSX conf)
* Panagiotis Cheilaris (serialize version)
* Jeroen Ooms (clang++ on MacOS ARM64)
* ZeroMQ Authors (source files in 'src/zmq_src/')

For the distribution of ZeroMQ that is shipped with pbdZMQ, you can find details of authorship and copyright in `inst/zmq_copyright/` of the pbdZMQ source tree, or under `zmq_copyright/` of a binary installation of pbdZMQ.

# Instructions

Before proceeding with the examples contained in this subdirectory,
please read all of the following carefully.  Failure to do so may
lead to confusion.


## Naming Conventions

Examples are given in the variety of interfaces avaialable in pbdZMQ.
The naming convention goes:

1. No preface is placed on the file name for the basic interface.
2. pyzmq_ prefaces the file name if the example uses the PyZMQ-like R6
   interface (see help("pyzmq") for details).
3. rzmq_ prefaces the file name if the example uses the rzmq-like
   interface (see help("rzmq") for details).


## Running An Example

Each of these examples should be executed in the following way:

1. Start the server by running server.r IN BATCH.  You can do this by
   executing `Rscript server.r` from a terminal.
2. Start the client by sourcing client.r INTERACTIVELY.  You can do this
   by starting an R session (Rstudio, wingui, R.app, ...) and running
   `source client.r`.
3. Pass commands from client to server by executing 
   `sendrecv(socket, "1+1")` from the client (replace "1+1" with the
   command of your choice, as a string).
4. Shut down the server by executing `sendrecv(socket, "EXIT")` on
   the client.

See the package vignette for more details.

### Multiple part server.
# SHELL> Rscript mpserver.r &
# SHELL> Rscript mpclient.r

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")

### Send and receive 5 times.
ret <- zmq.recv.multipart(responder, unserialize = TRUE)
parts <- as.list(rep("World", 5))
zmq.send.multipart(responder, parts)
for(i in 1:5) cat(ret[[i]])

### Finish.
zmq.close(responder)
zmq.ctx.destroy(context)

### Weather updates client as in the ZeroMQ guide.
# SHELL> Rscript wuserver.r &
# SHELL> Rscript wuclient.r
# SHELL> rm weather.ipc

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
cat("Collecting updates from weather server ...\n");
context <- zmq.ctx.new()
subscriber <- zmq.socket(context, ZMQ.ST()$SUB)
zmq.connect(subscriber, "tcp://localhost:5556")

### Ask for four zip codes.
filters <- c("50011", "37831", "37996", "20993")
for(i in 1:length(filters)){
  cat("Subscribe zipcode", filters[i], "\n")
  zmq.setsockopt(subscriber, ZMQ.SO()$SUBSCRIBE, filters[i])
}

### Process weather updates.
N.updates <- 30
temperature <- vector(mode = "list", length = 4)
while(any(sapply(temperature, length) < N.updates)){
  string <- zmq.recv(subscriber)
  msg <- strsplit(string$buf, " ")[[1]]  # c(zipcode, temperature, humidity)
  # cat(msg, "\n")
  id <- which(msg[1] == filters)
  temperature[[id]] <- c(temperature[[id]], as.integer(msg[2]))

  if(length(temperature[[id]]) == N.updates){
    ### No need to unsubscribe but demo the way how to do it.
    cat("Unsubscribe zipcode", filters[id], "\n", sep = " ");
    zmq.setsockopt(subscriber, ZMQ.SO()$UNSUBSCRIBE, filters[id])
  }
}
avg.temperature <- sapply(temperature, mean)

### Print average temperature for asked zip codes.
for(i in 1:length(filters)){
  cat("Average temperature for zipcode", filters[i],
      "was", avg.temperature[i], "\n")
}

### Finish.
zmq.close(subscriber)
zmq.ctx.destroy(context)

### Weather updates server as in the ZeroMQ guide.
# SHELL> Rscript wuserver.r &
# SHELL> Rscript wuclient.r
# SHELL> rm weather.ipc

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
publisher <- zmq.socket(context, ZMQ.ST()$PUB)
zmq.bind(publisher, "tcp://*:5556")
if(.Platform$OS.type != "windows"){  # Windows does not support ipc.
  zmq.bind(publisher, "ipc://weather.ipc")
}

### Send message to all subscribers.
cat("Press Ctrl+C or Esc to stop wuserver.\n")
zipcodes <- c(50011, 37831, 37996, 20993, sample.int(100000, size = 5))
while(TRUE){
  zipcode <- sample(zipcodes, 1)
  temperature <- sample.int(215, size = 1) - 80
  humidity <- sample.int(50, size = 1) + 10
  ret <- sprintf("%05d %d %d", zipcode, temperature, humidity)
  zmq.send(publisher, ret)
  Sys.sleep(runif(1, 0.1, 0.5))
}

### Finish.
zmq.close(publisher)
zmq.ctx.destroy(context)

### Remove ipc file if exists.
if(file.exists("weather.ipc")){
  file.remove("weather.ipc")
}

if (interactive())
  cat("WARNING: You are encouraged to run the server in batch; you can do so by executing 'Rscript server.r' from a terminal.\n\n")

library(pbdZMQ)
ctxt <- zmq.ctx.new()
socket <- zmq.socket(ctxt, ZMQ.ST()$REP)
zmq.bind(socket, "tcp://*:55555")


while(TRUE)
{
  cat("Client command:  ")
  msg <- zmq.msg.recv(socket)
  cat(msg, "\n")
  
  if (msg == "EXIT")
    break
  
  result <- eval(parse(text=msg))
  
  zmq.msg.send(result, socket)
}

zmq.msg.send("shutting down!", socket)

zmq.close(socket)
zmq.ctx.destroy(ctxt)

### Task sink from two workers as in the ZeroMQ guide.
# SHELL> Rscript taskwork.r &
# SHELL> Rscript taskvent.r; Rscript tasksink.r
### Remember to kill two worker processors at the end, such as
# SHELL> ps -x|grep "file=task.*\.r"|sed "s/\(.*\) pts.*/\1/"|xargs kill -9

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.bind(receiver, "tcp://*:5558")

### Wait for start of batch.
string <- zmq.recv(receiver)

### Process 100 confirmations.
start.time <- Sys.time()
for(i in 1:50){
  string <- zmq.recv(receiver)
  cat(i, ":", string$buf, "\n")
}
cat("Total elapsed time:", Sys.time() - start.time, "\n")

### Finish.
zmq.close(receiver)
zmq.ctx.destroy(context)

### Iterative pollers
# SHELL> Rscript iter_try.r

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")

### Process messages from the socket.
for(i in 1:5){
  cat("Press Ctrl+C or Esc to stop iter_script ... ", i, "\n", sep = "")
  aa <- tryCatch(zmq.poll(c(receiver), c(ZMQ.PO()$POLLIN)),
                 interrupt = function(c){ c$ret$pollret })
  print(aa)
}

### Finish.
zmq.close(receiver)
zmq.ctx.destroy(context)

### Hello world server as in the ZeroMQ guide.
# SHELL> Rscript hwserver.r &
# SHELL> Rscript hwclient.r

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")

### Send and receive 5 times.
for(i.res in 1:5){
  buf <- zmq.recv(responder, 10L)
  cat(buf$buf, "\n")
  Sys.sleep(0.5)
  zmq.send(responder, "World")
}

### Finish.
zmq.close(responder)
zmq.ctx.destroy(context)

if (!interactive())
  stop("Please run the client interactively; you can do so by executing 'source(\"client.r\")' from an interactive R session.")

library(pbdZMQ)
ctxt <- zmq.ctx.new()
socket <- zmq.socket(ctxt, ZMQ.ST()$REQ)
zmq.connect(socket, "tcp://localhost:55555")

sendrecv <- function(socket, data)
{
  zmq.msg.send(data, socket)
  zmq.msg.recv(socket)
}

cat("Send commands to the server using sendrecv().\nFor example, 'sendrecv(socket, \"1+1\")'.\nSend 'sendrecv(socket, \"EXIT\")' to shut down the server.\n\n")

### Multiple part client.
# SHELL> Rscript mpserver.r &
# SHELL> Rscript mpclient.r

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")

### Send and receive 5 times.
parts <- lapply(1:5, function(i.req){ paste("Sending Hello ", i.req, "\n") })
zmq.send.multipart(requester, parts)
ret <- zmq.recv.multipart(requester, unserialize = TRUE)
print(ret)

### Finish.
zmq.close(requester)
zmq.ctx.destroy(context)

### Multiple socket reader as in the ZeroMQ guide.
# SHELL> Rscript wuserver.r &
# SHELL> Rscript taskvent.r &
# SHELL> Rscript msreader.r
# SHELL> rm weather.ipc

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")
subscriber <- zmq.socket(context, ZMQ.ST()$SUB)
zmq.connect(subscriber, "tcp://localhost:5556")
zmq.setsockopt(subscriber, ZMQ.SO()$SUBSCRIBE, "20993")

### Process messages from both sockets.
cat("Press Ctrl+C or Esc to stop msreader.\n")
# while(TRUE){
  # while(TRUE){
  for(i in 1:5){
    ret <- zmq.recv(receiver)
    if(ret$len != -1){
      cat("task ventilator:", ret$buf, "\n")
    } else{
      break
    }
  }

  # while(TRUE){
  for(i in 1:5){
    ret <- zmq.recv(subscriber)
    if(ret$len != -1){
      cat("weather update:", ret$buf, "\n")
    } else{
      break
    }
  }

#   Sys.sleep(1/1000)
# }

### Finish.
zmq.close(receiver)
zmq.close(subscriber)
zmq.ctx.destroy(context)

if (!interactive())
  stop("Please run the client interactively; you can do so by executing 'source(\"client.r\")' from an interactive R session.")

library(pbdZMQ)
ctxt <- init.context()
socket <- init.socket(ctxt, "ZMQ_REQ")
connect.socket(socket, "tcp://localhost:55555")

sendrecv <- function(socket, data)
{
  send.socket(socket, data)
  receive.socket(socket)
}

cat("Send commands to the server using sendrecv().  For example, 'sendrecv(socket, \"1+1\")'.  Send 'sendrecv(socket, \"EXIT\")' to shut down the server.\n\n")

suppressMessages(library(pbdZMQ, quietly = TRUE))

context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")

a1 <- zmq.poll(c(receiver), ZMQ.PO()$POLLIN)
print(a1)

a2 <- tryCatch(zmq.poll(c(receiver), ZMQ.PO()$POLLIN),
       interrupt = function(c) return(9),
       error = function(c) return(99))
print(a2)

a3 <- tryCatch(zmq.poll(c(receiver), ZMQ.PO()$POLLIN,
                       MC = ZMQ.MC(check.eintr = TRUE)),
        interrupt = function(c) return(9),
        error = function(c) return(99))
print(a3)

a4 <- zmq.poll(c(receiver), ZMQ.PO()$POLLIN,
               MC = ZMQ.MC(check.eintr = TRUE))
print(a4)
print("This message should not be reached.")

if (interactive())
  cat("WARNING: You are encouraged to run the server in batch; you can do so by executing 'Rscript server.r' from a terminal.\n\n")

library(pbdZMQ)
ctxt <- init.context()
socket <- init.socket(ctxt, "ZMQ_REP")
bind.socket(socket, "tcp://*:55555")


while(TRUE)
{
  cat("Client command:  ")
  msg <- receive.socket(socket)
  cat(msg, "\n")

  if (msg == "EXIT")
    break
  
  result <- eval(parse(text=msg))

  send.socket(socket, result)
}

send.socket(socket, "shutting down!")

### Hello world client as in the ZeroMQ guide.
# SHELL> Rscript hwserver.r &
# SHELL> Rscript hwclient.r

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")

### Send and receive 5 times.
for(i.req in 1:5){
  cat("Sending Hello ", i.req, "\n")
  zmq.send(requester, "Hello")
  buf <- zmq.recv(requester, 10L)
  cat("Received World ", i.req, "\n")
} 

### Finish.
zmq.close(requester)
zmq.ctx.destroy(context)

### Multiple socket reader as in the ZeroMQ guide.
# SHELL> Rscript wuserver.r &
# SHELL> Rscript taskvent.r &
# SHELL> Rscript mspoller.r
# SHELL> rm weather.ipc

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")
subscriber <- zmq.socket(context, ZMQ.ST()$SUB)
zmq.connect(subscriber, "tcp://localhost:5556")
zmq.setsockopt(subscriber, ZMQ.SO()$SUBSCRIBE, "20993")

### Process messages from both sockets.
cat("Press Ctrl+C or Esc to stop mspoller.\n")
i.rec <- 0
i.sub <- 0
while(TRUE){
  ### Set poller.
  poller <- zmq.poll(c(receiver, subscriber),
                     c(ZMQ.PO()$POLLIN, ZMQ.PO()$POLLIN))

  ### Check receiver.
  if(bitwAnd(zmq.poll.get.revents(1), ZMQ.PO()$POLLIN)){
    ret <- zmq.recv(receiver)
    if(ret$len != -1){
      cat("task ventilator:", ret$buf, "at", i.rec, "\n")
      i.rec <- i.rec + 1
    }
  }

  ### Check subscriber.
  if(bitwAnd(zmq.poll.get.revents(2), ZMQ.PO()$POLLIN)){
    ret <- zmq.recv(subscriber)
    if(ret$len != -1){
      cat("weather update:", ret$buf, "at", i.sub, "\n")
      i.sub <- i.sub + 1
    }
  }

  if(i.rec >= 5 & i.sub >= 5){
    break
  }

  Sys.sleep(runif(1, 0.5, 1))
}

### Finish.
zmq.poll.free()
zmq.close(receiver)
zmq.close(subscriber)
zmq.ctx.destroy(context)

### Task ventilator send to two workers as in the ZeroMQ guide.
# SHELL> Rscript taskwork.r &
# SHELL> Rscript taskvent.r; Rscript tasksink.r
### Remember to kill two worker processors at the end, such as
# SHELL> ps -x|grep "file=task.*\.r"|sed "s/\(.*\) pts.*/\1/"|xargs kill -9

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
sender <- zmq.socket(context, ZMQ.ST()$PUSH)
zmq.bind(sender, "tcp://*:5557")
sink <- zmq.socket(context, ZMQ.ST()$PUSH)
zmq.connect(sink, "tcp://localhost:5558")

### Send sink.
readline("Press Enter when workers are ready:")
cat("Sending tasks to workers ...\n")
zmq.send(sink, "0")

### Send 100 tasks.
set.seed(1234)
total.msec <- 0
for(i in 1:100){
  workload <- as.integer(runif(1, 1, 30))
  total.msec <- total.msec + workload
  string <- sprintf("%d", workload)
  zmq.send(sender, string)
  Sys.sleep(runif(1, 3, 5))
}
cat("Total expected cost: ", total.msec, " msec\n", sep = "")

### Finish.
zmq.close(sink)
zmq.close(sender)
zmq.ctx.destroy(context)

### Task workers as in the ZeroMQ guide.
# SHELL> Rscript taskwork.r &
# SHELL> Rscript taskvent.r; Rscript tasksink.r
### Remember to kill two worker processors at the end, such as
# SHELL> ps -x|grep "file=task.*\.r"|sed "s/\(.*\) pts.*/\1/"|xargs kill -9

suppressMessages(library(pbdZMQ, quietly = TRUE))

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")
sender <- zmq.socket(context, ZMQ.ST()$PUSH)
zmq.connect(sender, "tcp://localhost:5558")

### Process tasks forever.
while(TRUE){
  string <- zmq.recv(receiver)
  # cat(string$buf, "\n")
  Sys.sleep(as.integer(string$buf) / 1000)
  zmq.send(sender, paste("sleep", string$buf, "sec at", Sys.getpid()))
}

### Finish.
zmq.close(receiver)
zmq.close(sender)
zmq.ctx.destroy(context)

#' Context Functions
#' 
#' Context functions
#' 
#' \code{zmq.ctx.new()} initializes a ZMQ context for starting communication.
#' 
#' \code{zmq.ctx.destroy()} terminates the context for stopping communication.
#' 
#' @param ctx 
#' a ZMQ context
#' 
#' @return 
#' \code{zmq.ctx.new()} returns an R external pointer (\code{ctx})
#' generated by ZMQ C API pointing to a context if successful, otherwise
#' returns an R \code{NULL}.
#' 
#' \code{zmq.ctx.destroy()} returns 0 if successful, otherwise returns -1 and
#' sets \code{errno} to either \code{EFAULT} or \code{EINTR}.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.socket}()}, \code{\link{zmq.close}()},
#' \code{\link{zmq.bind}()}, \code{\link{zmq.connect}()}.
#' @rdname a0_d_context
#' @name Context Functions
NULL



#' @rdname a0_d_context
#' @export
zmq.ctx.new <- function(){
  ret <- .Call("R_zmq_ctx_new", PACKAGE = "pbdZMQ")
  ### Users are responsible to take care free and gc.
  # reg.finalizer(ret, zmq.ctx.destroy, TRUE)
  ret
}



#' @rdname a0_d_context
#' @export
zmq.ctx.destroy <- function(ctx){
  .Call("R_zmq_ctx_destroy", ctx, PACKAGE = "pbdZMQ")
  invisible()
}

### Modified from Rserve/src/install.libs.R
### For libs
files <- c("pbdZMQ.so", "pbdZMQ.so.dSYM", "pbdZMQ.dylib", "pbdZMQ.dll",
           "symbols.rds",
           "libzmq.so", "libzmq.so.dSYM", "libzmq.dll")
files <- files[file.exists(files)]
lib.osx <- list.files(pattern = "libzmq\\.(.*)\\.dylib")
files <- c(files, lib.osx)

if(length(files) > 0){
  libsarch <- if (nzchar(R_ARCH)) paste("libs", R_ARCH, sep='') else "libs"
  dest <- file.path(R_PACKAGE_DIR, libsarch)
  dir.create(dest, recursive = TRUE, showWarnings = FALSE)
  file.copy(files, dest, overwrite = TRUE, recursive = TRUE)

  ### For Mac OSX and when "internal ZMQ" is asked.
  ### Overwrite RPATH from the shared library installed to the destination.
  if(Sys.info()[['sysname']] == "Darwin" && length(lib.osx) > 0){
    cmd.int <- system("which install_name_tool", intern = TRUE)
    cmd.ot <- system("which otool", intern = TRUE) 
    fn.pbdZMQ.so <- file.path(dest, "pbdZMQ.so")

    if(length(lib.osx) != 1){
      print(lib.osx)
      stop("More than one libzmq.*.dylib are found.")
    } else{
      fn.libzmq.dylib <- file.path(dest, lib.osx)

      if(file.exists(fn.pbdZMQ.so) && file.exists(fn.libzmq.dylib)){
        ### For pbdZMQ.so
        rpath <- system(paste(cmd.ot, " -L ", fn.pbdZMQ.so, sep = ""),
                        intern = TRUE)
        cat("\nBefore install_name_tool (install.libs.R & pbdZMQ.so):\n")
        print(rpath)

        ### R-3.6.* does not like absolute path and staged installation has
        ### it's own way (see "tools/R/install.R") to overwrite the id.
        if(getRversion() < "3.6.0"){
          cmd <- paste(cmd.int, " -id ", fn.pbdZMQ.so, " ",
                       fn.pbdZMQ.so, sep = "")
          cat("\nIn install_name_tool (install.libs.R & pbdZMQ.so & id):\n")
          print(cmd) 
          system(cmd)
        }

        str.lib <- paste("zmq/lib/", lib.osx, sep = "")
        org <- file.path(getwd(), str.lib)
        cmd <- paste(cmd.int, " -change ", org, " @loader_path/", lib.osx, " ",
                     fn.pbdZMQ.so, sep = "")
        cat("\nIn install_name_tool (install.libs.R & pbdZMQ.so & path):\n")
        print(cmd) 
        system(cmd)

        rpath <- system(paste(cmd.ot, " -L ", fn.pbdZMQ.so, sep = ""),
                        intern = TRUE)
        cat("\nAfter install_name_tool (install.libs.R & pbdZMQ.so):\n")
        print(rpath)

        ### For libzmq.dylib
        rpath <- system(paste(cmd.ot, " -L ", fn.libzmq.dylib, sep = ""),
                        intern = TRUE)
        cat("\nBefore install_name_tool (install.libs.R & libzmq.dylib):\n")
        print(rpath)

        ### R-3.6.* does not like absolute path because of staged installation.
        if(getRversion() < "3.6.0"){
          cmd <- paste(cmd.int, " -id ", fn.libzmq.dylib, " ",
                       fn.libzmq.dylib, sep = "")
        } else{
          cmd <- paste(cmd.int, " -id ", lib.osx, " ",
                       fn.libzmq.dylib, sep = "")
        }
        cat("\nIn install_name_tool (install.libs.R & libzmq.dylib & id):\n")
        print(cmd) 
        system(cmd)

        rpath <- system(paste(cmd.ot, " -L ", fn.libzmq.dylib, sep = ""),
                        intern = TRUE)
        cat("\nAfter install_name_tool (install.libs.R & libzmq.dylib):\n")
        print(rpath)
      }
    }
  }
}

### For etc
file <- "Makeconf"
if(file.exists(file)){
  etcarch <- if (nzchar(R_ARCH)) paste("etc", R_ARCH, sep='') else "etc"
  dest <- file.path(R_PACKAGE_DIR, etcarch)
  dir.create(dest, recursive = TRUE, showWarnings = FALSE)
  file.copy(file, dest, overwrite = TRUE)
}

### For zmq and pbdZMQ include
dir.zmq <- "./zmq"
if(file.exists(dir.zmq)){
  libarch <- if (nzchar(R_ARCH)) paste("zmq", R_ARCH, sep='') else "zmq"
  dest <- file.path(R_PACKAGE_DIR, libarch)
  dir.create(dest, recursive = TRUE, showWarnings = FALSE)
  files <- paste(dir.zmq, "/include" , sep = "")
  file.copy("./R_zmq.h", files)
  file.copy(files, dest, overwrite = TRUE, recursive = TRUE)
}

### This file is only called by
###   "pbd*/src/Makevars.in" or "pbd*/src/Makevar.win"
### to find the default configurations from
###   "pbd*/etc${R_ARCH}/Makconf".

get.path.lib <- function(arch, fn.in, debug = FALSE){
  ### For the nm outputs to check 32- and 64-bits libraries.
  if(arch == "/i386"){
    n.zero <- 8
  } else if(arch == "/x64"){
    n.zero <- 16 
  } else{
    stop(paste(arch, " is not found.", sep = ""))
  }

  ### Find which path gcc.exe is located.
  path.env <- Sys.getenv("PATH")
  path <- unlist(strsplit(path.env, ";"))
  check.gcc <- FALSE
  for(i.path in 1:length(path)){
    dir.gcc <- gsub("\\\\", "/", path[i.path])
    path.gcc <- paste(dir.gcc, "./gcc.exe", sep = "")
    check.gcc <- file.exists(path.gcc)
    if(check.gcc){
      break
    }
  }

  ### Get root for the Rtools
  if(!check.gcc){
    if(debug){
      print(path)
    }
    stop("gcc is not found.")
  } else{
    if(debug){
      print(path.gcc)
    }
    path.rtools <- gsub("/bin.*", "", path.gcc)
  }

  ### Check libraries.
  # fn.in <- "libiphlpapi.a"
  cmd <- paste("find ", path.rtools, " | grep '", fn.in, "$'", sep = "")
  libs <- shell(cmd, intern = TRUE, ignore.stderr = TRUE)
  if(debug){
    print(libs)
  }
  if(length(libs) == 0){
    stop(paste(fn.in, " is not found in ", path.rtools, sep = ""))
  } else{
    check.bits <- FALSE
    for(i.lib in 1:length(libs)){
      fn.out <- libs[i.lib]
      cmd <- paste("nm ", fn.out, " | grep '^0' | head -1 | sed -e 's/ .*//'",
                   sep = "")
      bits <- shell(cmd, intern = TRUE, ignore.stderr = TRUE)
      zeros <- bits[1]
      if(debug){
        print(zeros)
      } 
      if(nchar(zeros) == n.zero){
        check.bits <- TRUE
        break
      }
    }

    if(check.bits){
      fn.in <- gsub("\\+", "\\\\+", fn.in)
      path.lib <- gsub(paste("/", fn.in, sep = ""), "/", fn.out)
      if(debug){
        print(path.lib)
      }
    } else{
      stop("check.bits is neither 8 nor 16.")
    }
  }

  path.lib
} # End of get.path.lib().

get.path.lib.330 <- function(arch, binpref, fn.in, debug = FALSE){
  ### For the nm outputs to check 32- and 64-bits libraries.
  if(arch == "/i386"){
    arch <- gsub("^.", "", arch)
  } else if(arch == "/x64"){
    arch <- gsub("^.", "", arch)
  } else{
    stop(paste(arch, " is not found.", sep = ""))
  }

  ### Find which path gcc.exe is located.
  if(binpref != ""){
    path.gcc <- paste(binpref, "gcc.exe", sep = "")
  } else{
    ### For Rtools33 or newer version.
    ### This dose not work.
    cmd <- paste("R --arch ", arch, " CMD config CC", sep = "")
    path.gcc <- shell(cmd, intern = TRUE, ignore.stderr = TRUE)
    path.gcc <- paste(path.gcc, ".exe", sep = "")
  }
  check.gcc <- file.exists(path.gcc)

  ### Get root for the Rtools
  if(debug){
    print(path.gcc)
  }
  path.rtools <- gsub("/bin.*", "", path.gcc)

  ### Check libraries.
  # fn.in <- "libiphlpapi.a"
  cmd <- paste("find ", path.rtools, " | grep '", fn.in, "$'", sep = "")
  libs <- shell(cmd, intern = TRUE, ignore.stderr = TRUE)
  fn.out <- libs[1]
  fn.in <- gsub("\\+", "\\\\+", fn.in)
  path.lib <- gsub(paste("/", fn.in, sep = ""), "/", fn.out)
  if(debug){
    print(path.lib)
  }

  path.lib
} # End of get.path.lib().

### For libiphlpapi.a, librpcrt4.a, and libws2_32.a
### C:\Rtools32\gcc-4.6.3\i686-w64-mingw32\lib
### C:\Rtools32\gcc-4.6.3\i686-w64-mingw32\lib64
### C:\Rtools33\mingw_32\i686-w64-mingw32\lib
### C:\Rtools33\mingw_64\x86_64-w64-mingw32\lib
get.mingw.lib <- function(arch = '', binpref = '', debug = FALSE){
  if(getRversion() < '3.3.0'){
    fn.in <- "libiphlpapi.a"
    path.lib <- get.path.lib(arch, fn.in, debug = debug)

    fn.in <- "librpcrt4.a"
    path.fn <- paste(path.lib, fn.in, sep = "")
    check.fn <- file.exists(path.fn)
    if(!check.fn){
      stop(paste(path.fn, " is not found.", sep = ""))
    }

    fn.in <- "libws2_32.a"
    path.fn <- paste(path.lib, fn.in, sep = "")
    check.fn <- file.exists(path.fn)
    if(!check.fn){
      stop(paste(path.fn, " is not found.", sep = ""))
    }

    ### Cat back to "Makefile.win".
    cat(path.lib)
  }

  invisible()
} # End of get.mingw.lib().

### For libstdc++.a
### C:\Rtools32\gcc-4.6.3\lib
### C:\Rtools32\gcc-4.6.3\lib64
### C:\Rtools33\mingw_32\lib\gcc\i686-w64-mingw32\4.9.3
### C:\Rtools33\mingw_64\lib\gcc\x86_64-w64-mingw32\4.9.3
get.stdcxx.lib <- function(arch = '', binpref = '', debug = FALSE){
  if(getRversion() < '3.3.0'){
    fn.in <- "libstdc++.a"
    path.lib <- get.path.lib(arch, fn.in, debug = debug)

    ### Cat back to "Makefile.win".
    cat(path.lib)
  }

  invisible()
} # End of get.stdcxx.lib().

#' Message Functions
#' 
#' Message functions
#' 
#' \code{zmq.msg.send()} sends an R message.
#' 
#' \code{zmq.msg.recv()} receives an R message.
#' 
#' @param rmsg 
#' an R message
#' @param socket 
#' a ZMQ socket
#' @param flags 
#' a flag for method of send and receive
#' @param serialize 
#' if serialize the \code{rmsg}
#' @param unserialize 
#' if unserialize the received R message
#' @param serialversion
#' NULL or numeric; the workspace format version to use when serializing.
#' NULL specifies the current default version. The only other supported
#' values are 2 and 3
#' 
#' @return \code{zmq.msg.send()} returns 0 if successful, otherwise returns -1
#' and sets \code{errno} to \code{EFAULT}.
#' 
#' \code{zmq.msg.recv()} returns the message if successful, otherwise returns
#' -1 and sets \code{errno} to \code{EFAULT}.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Using request-reply pattern.
#' 
#' ### At the server, run next in background or the other window.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' responder <- zmq.socket(context, ZMQ.ST()$REP)
#' zmq.bind(responder, "tcp://*:5555")
#' buf <- zmq.msg.recv(responder)
#' set.seed(1234)
#' ret <- rnorm(5)
#' print(ret)
#' zmq.msg.send(ret, responder)
#' zmq.close(responder)
#' zmq.ctx.destroy(context)
#' 
#' 
#' ### At a client, run next in foreground.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' requester <- zmq.socket(context, ZMQ.ST()$REQ)
#' zmq.connect(requester, "tcp://localhost:5555")
#' zmq.msg.send(NULL, requester)
#' ret <- zmq.msg.recv(requester)
#' print(ret)
#' zmq.close(requester)
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.send}()}, \code{\link{zmq.recv}()}.
#' @rdname a2_message
#' @name Message Function
NULL



zmq.msg.init <- function(){
  .Call("R_zmq_msg_init", PACKAGE = "pbdZMQ")
}



zmq.msg.close <- function(msg.t){
  .Call("R_zmq_msg_close", msg.t, PACKAGE = "pbdZMQ")
}



#' @rdname a2_message
#' @export
zmq.msg.send <- function(rmsg, socket, flags = ZMQ.SR()$BLOCK,
                         serialize = TRUE, serialversion = NULL){
  if(serialize){
    rmsg <- serialize(rmsg, NULL, version = serialversion)
  }
  ret <- .Call("R_zmq_msg_send", rmsg, socket, as.integer(flags), PACKAGE = "pbdZMQ")
  invisible(ret)
}



#' @rdname a2_message
#' @export
zmq.msg.recv <- function(socket, flags = ZMQ.SR()$BLOCK,
                         unserialize = TRUE){
  rmsg <- .Call("R_zmq_msg_recv", socket, as.integer(flags), PACKAGE = "pbdZMQ")
  if(unserialize && is.raw(rmsg)){
    rmsg <- unserialize(rmsg)
  }
  rmsg
}

#' Send Receive Multiple Raw Buffers
#' 
#' Send and receive functions for multiple raw buffers
#' 
#' \code{zmq.send.multipart()} is a high level R function to send multiple
#' raw messages \code{parts} at once.
#' 
#' \code{zmq.recv.multipart()} is a high level R function to receive multiple
#' raw messages at once.
#' 
#' @param socket 
#' a ZMQ socket
#' @param parts
#' a vector of multiple buffers to be sent
#' @param serialize,unserialize
#' if serialize/unserialize the received multiple buffers
#' 
#' @return \code{zmq.send.multipart()} returns.
#'
#' \code{zmq.recv.multipart()} returns.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Using request-reply pattern.
#' 
#' ### At the server, run next in background or the other window.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' responder <- zmq.socket(context, ZMQ.ST()$REP)
#' zmq.bind(responder, "tcp://*:5555")
#' 
#' ret <- zmq.recv.multipart(responder, unserialize = TRUE)
#' parts <- as.list(rep("World", 5))
#' zmq.send.multipart(responder, parts)
#' for(i in 1:5) cat(ret[[i]])
#' 
#' zmq.close(responder)
#' zmq.ctx.destroy(context)
#' 
#' ### At a client, run next in foreground.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' requester <- zmq.socket(context, ZMQ.ST()$REQ)
#' zmq.connect(requester, "tcp://localhost:5555")
#' 
#' parts <- lapply(1:5, function(i.req){ paste("Sending Hello ", i.req, "\n") })
#' zmq.send.multipart(requester, parts)
#' ret <- zmq.recv.multipart(requester, unserialize = TRUE)
#' print(ret)
#' 
#' zmq.close(requester)
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
#' @rdname b2_sendrecv_multipart
#' @name Send Receive Multiple Raw Buffers
NULL



#' @rdname b2_sendrecv_multipart
#' @export
zmq.send.multipart <- function(socket, parts, serialize = TRUE){
  for(i.part in 1:(length(parts) - 1)){
    zmq.msg.send(parts[[i.part]], socket, flags = ZMQ.SR()$SNDMORE,
                 serialize = serialize)
  }
  zmq.msg.send(parts[[length(parts)]], socket, flags = ZMQ.SR()$BLOCK,
               serialize = serialize)

  invisible()
}



#' @rdname b2_sendrecv_multipart
#' @export
zmq.recv.multipart <- function(socket, unserialize = TRUE){
  ret <- list() 
  i.part <- 1
  ret[[i.part]] <- zmq.msg.recv(socket, flags = ZMQ.SR()$BLOCK,
                                unserialize = unserialize)
  opt.val <- zmq.getsockopt(socket, ZMQ.SO()$RCVMORE, 0L)

  while(opt.val == 1){
    i.part <- i.part + 1
    ret[[i.part]] <- zmq.msg.recv(socket, flags = ZMQ.SR()$BLOCK,
                                  unserialize = unserialize)
    opt.val <- zmq.getsockopt(socket, ZMQ.SO()$RCVMORE, 0L)
  }

  ret
}

#' Overwrite rpath of linked shared library in osx
#' 
#' Overwrite rpath of linked shared library
#' (e.g. \code{JuniperKernel/libs/JuniperKernel.so}
#' in osx only.
#' Typically, it is called by \code{.onLoad()} to update rpath if
#' \code{pbdZMQ} or \code{pbdZMQ/libs/libzmq.*.dylib} was moved to
#' a personal directory
#' (e.g. the binary package was installed to a none default path).
#' The commands \code{otool} and \code{install_name_tool} are required.
#' Permission may be needed (e.g. \code{sudo}) to overwrite the shared
#' library.
#' 
#' @param mylib
#' the path where \code{mypkg} was installed (default \code{NULL} that will
#' search from R's path)
#' @param mypkg
#' the package for where \code{mypkg.so} will be checked or updated
#' @param linkingto
#' the package for where \code{libshpkg*.dylib} is located
#' @param shlib
#' name of shlib to be searched for
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Called by .onLoad() within "JuniperKernel/R/zzz.R"
#' overwrite.shpkg.rpath(mypkg = "JuniperKernel",
#'                       linkingto = "pbdZMQ",
#'                       shlib = "zmq")
#' }
#' 
#' @keywords compile
#' @rdname zz_overwrite_shpkg
#' @name Overwrite shpkg
NULL


#' @rdname zz_overwrite_shpkg
#' @export
overwrite.shpkg.rpath <- function(mylib = NULL,
    mypkg = "JuniperKernel", linkingto = "pbdZMQ", shlib = "zmq"){
  if(Sys.info()[['sysname']] == "Darwin"){
    cmd.ot <- system("which otool", intern = TRUE)

    ### Find mylib if NULL.
    if(is.null(mylib)){
      mylib <- tools::file_path_as_absolute(system.file(package = mypkg)) 
    }

    ### Get rpath from mypkg's shared library.
    fn.so <- paste(mylib, "/", mypkg, "/libs/", mypkg, ".so", sep = "")
    rpath <- system(paste(cmd.ot, " -L ", fn.so, sep = ""), intern = TRUE)

    ### Get the installed dylib from mypkg's rpath.
    ptn1 <- paste("^\\t(.*/", linkingto, "/libs/lib", shlib, ".*\\.dylib) .*$",
                  sep = "")
    i.rpath <- grep(ptn1, rpath)

    ### 0 means external shlib was linked.
    ### 1 means internal shlib was built.
    if(length(i.rpath) == 1){
      fn.dylib <- gsub(ptn1, "\\1", rpath[i.rpath])

      ### Check if the file were moved to somewhere.
      if(!file.exists(fn.dylib)){
        ### Search the new location from R's library path.
        dn <- tools::file_path_as_absolute(
                system.file("./libs", package = linkingto)) 
        ptn2 <- paste("lib", shlib, ".*\\.dylib", sep = "")
        fn <- list.files(path = dn, pattern = ptn2)

        ### Install the new location to the mypkg's shared library.
        if(length(fn) == 1){
          cmd.int <- system("which install_name_tool", intern = TRUE)

          new.fn.dylib <- paste(dn, "/", fn, sep = "")
          cmd <- paste(cmd.int, " -change ", fn.dylib, " ", new.fn.dylib,
                       " ", fn.so,
                       sep = "")
          system(cmd)
        } else{
          stop("The internal shlib can not be identified in linkingto.")
        }
      }
    }
  }

  return(invisible())
} # End of overwrite.shpkg().

#' Utility Functions
#' 
#' Utility functions
#' 
#' \code{zmq.strerror()} gets ZeroMQ error message string.
#'
#' \code{zmq.version()} print current ZeroMQ version.
#' 
#' @param errno 
#' an integer for the error number
#' 
#' @return 
#' \code{zmq.strerror()} returns an R string containing ZeroMQ error
#' message.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' zmq.ctx.destroy(context)
#' zmq.strerror(0)
#' 
#' zmq.ctx.destroy(context) # Error since context is free.
#' zmq.strerror(14)
#' }
#' 
#' @keywords internal
#' @seealso \code{\link{zmq.ctx.new}()}, \code{\link{zmq.ctx.destroy}()},
#' \code{\link{zmq.socket}()}, \code{\link{zmq.close}()}.
#' @rdname xx_utility
#' @name Utility Functions
NULL



#' @rdname xx_utility
#' @export
zmq.strerror <- function(errno){
  .Call("R_zmq_strerror", as.integer(errno[1]), PACKAGE = "pbdZMQ")
}


#' @rdname xx_utility
#' @export
zmq.version <- function(){
  ret <- .Call("R_zmq_version", PACKAGE = "pbdZMQ")
  package_version(ret)
}

#' Shell Execution via cmd windows
#' 
#' This function is an extension to the \code{shell.exec}() which is a native
#' function of R
#' 
#' \code{shell.exec("a.txt")} will open a windows (notepad) to edit the file
#' \code{a.txt} in windows system. However, the notepad will block the (parent)
#' active R windows, i.e. \code{SW.cmd = 5} as \code{SH_SHOW} by default.
#' 
#' The \code{shellexec.wcc("a.txt", SW.cmd = 7L)} will open the notepad, but in
#' a minimized window. Therefore, there is no blocking to the active R windows.
#' See the website in the references section to see more options to control the
#' behavior of new windows. Possible choices are
#' 
#' \code{SW_SHOW (5)}: Activates the window and displays it in its current size
#' and position.
#' 
#' \code{SW_SHOWMINIMIZED (2)}: Activates the window and displays it as a
#' minimized window.
#' 
#' \code{SW_SHOWMINNOACTIVE (7)}: Displays the window as a minimized window.
#' The active window remains active.
#' 
#' \code{SW_SHOWNA (8)}: Displays the window in its current state.  The active
#' window remains active.
#' 
#' @param file 
#' a file name as in \code{shell.exec}()
#' @param SW.cmd 
#' a SW_* command of showing windows
#' 
#' @return 
#' A new windows with certain applications depending on the association
#' of the input \code{file}.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' 
#' @references Microsoft, Windows Dev Center: Windows desktop applications >
#' Develop > Desktop technologies > Desktop Environment > The Windows Shell >
#' Shell Reference > Shell Functions > ShellExecute
#' 
#' \code{https://msdn.microsoft.com/en-us/library/windows/desktop/bb762153(v=vs.85).aspx}
#' 
#' @examples
#' \dontrun{
#' library(pbdZMQ, quietly = TRUE)
#' 
#' shellexec.wcc("a.txt", 5L)
#' }
#' 
#' @keywords internal
#' @seealso \code{shell.exec()}.
#' @rdname u0_shellexec.wcc
shellexec.wcc <- function(file, SW.cmd = 7L){
  if(length(file) == 1 && is.character(file)){
    fn.enc <- Encoding(file)
    if(fn.enc == "latin1"){
      fn.enc <- 1L
    } else if(fn.enc == "UTF-8"){
      fn.enc <- 2L
    } else if(fn.enc == "bytes"){
      fn.enc <- 3L
    } else if(fn.enc == "unknown"){
      fn.enc <- 4L
    }
    .Call("shellexec_wcc", file, as.integer(SW.cmd), as.integer(fn.enc),
          PACKAGE="pbdZMQ")
  } else{
    stop("file should be a character vector of length 1.")
  }
}

#' Poll Functions
#' 
#' Poll functions
#' 
#' \code{zmq.poll()} initials ZMQ poll items given  ZMQ \code{socket}'s
#' and ZMQ poll \code{type}'s. Both \code{socket} and \code{type} are
#' in vectors of the same length, while \code{socket} contains socket pointers
#' and \code{type} contains types of poll.
#' See \code{\link{ZMQ.PO}()} for the possible values of
#' \code{type}. ZMQ defines several poll types and utilize
#' them to poll multiple sockets.
#' 
#' \code{zmq.poll.free()} frees ZMQ poll structure memory internally.
#'
#' \code{zmq.poll.length()} obtains total numbers of ZMQ poll items.
#'
#' \code{zmq.poll.get.revents()} obtains revent types from ZMQ poll item by
#' the input index.
#' 
#' @param socket 
#' a vector of ZMQ sockets
#' @param type 
#' a vector of socket types corresponding to \code{socket} argument
#' @param timeout
#' timeout for poll, see ZeroMQ manual for details
#' @param index
#' an index of ZMQ poll items to obtain revents
#' @param MC 
#' a message control, see \code{\link{ZMQ.MC}()} for details
#' 
#' @return \code{zmq.poll()} returns a ZMQ code and an errno,
#' see ZeroMQ manual for details, no error/warning/interrupt in this
#' \code{R} function, but some error/warning/interrupt may catch by
#' the \code{C} function \code{zmq_poll()}.
#' @return \code{zmq.poll.length()} returns the total number of poll items
#' @return \code{zmq.poll.get.revents()} returns the revent type
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Using poll pattern.
#' ### See demo/mspoller.r for details.
#'
#' ### Run next in background or the other window.
#' SHELL> Rscript wuserver.r &
#' SHELL> Rscript taskvent.r &
#' SHELL> Rscript mspoller.r
#'
#' ### The mspoller.r has next.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' ### Initial.
#' context <- zmq.ctx.new()
#' receiver <- zmq.socket(context, ZMQ.ST()$PULL)
#' zmq.connect(receiver, "tcp://localhost:5557")
#' subscriber <- zmq.socket(context, ZMQ.ST()$SUB)
#' zmq.connect(subscriber, "tcp://localhost:5556")
#' zmq.setsockopt(subscriber, ZMQ.SO()$SUBSCRIBE, "20993")
#' 
#' ### Process messages from both sockets.
#' cat("Press Ctrl+C or Esc to stop mspoller.\n")
#' i.rec <- 0
#' i.sub <- 0
#' while(TRUE){
#'   ### Set poller.
#'   zmq.poll(c(receiver, subscriber),
#'            c(ZMQ.PO()$POLLIN, ZMQ.PO()$POLLIN))
#' 
#'   ### Check receiver.
#'   if(bitwAnd(zmq.poll.get.revents(1), ZMQ.PO()$POLLIN)){
#'     ret <- zmq.recv(receiver)
#'     if(ret$len != -1){
#'       cat("task ventilator:", ret$buf, "at", i.rec, "\n")
#'       i.rec <- i.rec + 1
#'     }
#'   }
#' 
#'   ### Check subscriber.
#'   if(bitwAnd(zmq.poll.get.revents(2), ZMQ.PO()$POLLIN)){
#'     ret <- zmq.recv(subscriber)
#'     if(ret$len != -1){
#'       cat("weather update:", ret$buf, "at", i.sub, "\n")
#'       i.sub <- i.sub + 1
#'     }
#'   }
#' 
#'   if(i.rec >= 5 & i.sub >= 5){
#'     break
#'   }
#' 
#'   Sys.sleep(runif(1, 0.5, 1))
#' }
#' 
#' ### Finish.
#' zmq.poll.free()
#' zmq.close(receiver)
#' zmq.close(subscriber)
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.recv}()}, \code{\link{zmq.send}()}.
#' @rdname b3_poll
#' @name Poll Functions
NULL



#' @rdname b3_poll
#' @export
zmq.poll <- function(socket, type, timeout = -1L, MC = ZMQ.MC()){
  if(length(socket) != length(type)){
    stop("socket and type are of different length.")
  }

  type <- as.integer(type)
  if(!all(type %in% 1:7)){
    stop("type should be integers in 1 to 7.")
  }

  zmq.poll.free()

  ret <- .Call("R_zmq_poll", socket, type, as.integer(timeout),
               as.logical(MC$check.eintr),
               PACKAGE = "pbdZMQ")
  return(invisible(ret))
}


#' @rdname b3_poll
#' @export
zmq.poll.free <- function(){
  ret <- .Call("R_zmq_poll_free", PACKAGE = "pbdZMQ")
  invisible(ret)
}

#' @rdname b3_poll
#' @export
zmq.poll.length <- function(){
  ret <- .Call("R_zmq_poll_length", PACKAGE = "pbdZMQ")
  invisible(ret)
}

#' @rdname b3_poll
#' @export
zmq.poll.get.revents <- function(index = 1L){
  if(index < 1){
    stop("index is a positive interger.")
  }
  ret <- .Call("R_zmq_poll_get_revents", as.integer(index - 1)[1],
               PACKAGE = "pbdZMQ")
  invisible(ret)
}

#' Random Port
#' 
#' Generate a valid, random TCP port.
#' 
#' By definition, a TCP port is an unsigned short, and so it can not
#' exceed 65535.  Additionally, ports in the range 1024 to 49151 are
#' (possibly) registered by ICANN for specific uses.
#' 
#' \code{random_port()} will simply generate a valid, non-registered
#' tcp port.  \code{random_unused_port()} will generate a port
#' that is available for socket connections.
#' 
#' \code{random_open_port()} finds a random port not already bound
#' to an endpoint.
#' 
#' @param min_port,max_port
#' The minimum/maximum value to be generated.  The minimum should not
#' be below 49152 and the maximum should not exceed 65536 (see
#' details).
#' @param max_tries
#' The maximum number of times a random port will be searched for.
#' 
#' @references
#' "The Ephemeral Port Range" by Mike Gleason.  
#' \url{https://www.ncftp.com/ncftpd/doc/misc/ephemeral_ports.html}
#' 
#' @author Drew Schmidt
#' 
#' @examples
#' random_port()
#' 
#' @importFrom stats runif
#' 
#' @rdname random_port
#' @export
random_port <- function(min_port=49152, max_port=65536)
{
  if (min_port < 49152)
    warning("non-recommended 'min_port' value; see ?random_port for details")
  if (max_port > 65536)
    warning("non-recommended 'max_port' value; see ?random_port for details")
  
  
  as.integer(runif(1, min_port, max_port+1))
}



#' @rdname random_port
#' @export
random_open_port <- function(min_port=49152, max_port=65536, max_tries=100)
{
  ctxt <- init.context()
  socket <- init.socket(ctxt, "ZMQ_REP")
  
  ret <- -1
  
  for (i in 1:max_tries)
  {
    port <- random_port(min_port=min_port, max_port=max_port)
    addr <- paste0("tcp://*:", port)
    
    catch <- tryCatch(zmq.bind(socket, addr), error=identity, warning=identity)
    if (!inherits(catch, "error") && !inherits(catch, "warning"))
    {
      ret <- port
      break
    }
  }
  
  rm(socket);rm(ctxt);
  invisible(gc())
  
  if (ret == -1)
    stop("No valid port could be found.")
  else
    return(ret)
}

#' Form an Address/Endpoint
#' 
#' A notationally convenient function for forming addresses/endpoints.
#' It's a simple wrapper around the \code{paste0()} function.
#' 
#' @param host
#' The host ip address or url.
#' @param port
#' A port; necessary for all transports except ipc.
#' @param transport
#' The transport protocol.  Choices are "inproc", "ipc", "tcp", and
#' "pgm"/"epgm" for local in-process (inter-thread), local 
#' inter-process, tcp, and pgm, respectively.
#' 
#' @return
#' An address, for use with pbdZMQ functions.
#' 
#' @author Drew Schmidt
#' 
#' @examples
#' address("localhost", 55555)
#' 
#' @seealso
#' \code{\link{zmq.bind}}
#' 
#' @export
address <- function(host, port, transport="tcp")
{
  transports <- c("tcp", "inproc", "ipc", "pgm", "epgm")
  transport <- tolower(transport)
  match.arg(transport, transports)
  
  if (transport == "ipc")
  {
    if (!missing(port))
      warning("Ignoring specified port for ipc transport.")
    
    paste0(transport, "://", host)
  }
  else
    paste0(transport, "://", host, ":", port)
}

#' The C-like ZeroMQ Interface
#' 
#' @description
#' The basic interface to ZeroMQ that somewhat models the C interface.
#' 
#' @details
#' A list of all functions for this interface is as follows:
#' 
#' \tabular{lll}{
#' \code{zmq.bind()} \tab \code{zmq.close()} \tab \code{zmqconnect()} \cr
#' \code{zmq.ctx.destroy()} \tab \code{zmq.ctx.new()} \tab \code{zmq.msg.recv()} \cr
#' \code{zmq.msg.send()} \tab \code{zmq.recv()} \tab \code{zmq.send()} \cr
#' \code{zmq.socket()} \tab \tab \cr
#' }
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @keywords zmq
#' @rdname xx_czmq_wrapper
#' @name C-like Wrapper Functions for ZeroMQ
NULL

#' All Wrapper Functions for rzmq
#' 
#' Wrapper functions for backwards compatibility with rzmq.  See vignette
#' for examples.
#' 
#' @details
#' \code{send.socket()}/\code{receive.socket()} send/receive messages over
#' a socket.  These are simple wrappers around \code{zmq.msg.send()} and
#' \code{zmq.msg.receive()}, respectively.
#' 
#' \code{init.context()} creates a new ZeroMQ context.  A useful wrapper
#' around \code{zmq.ctx.new()} which handles freeing memory for you, i.e.
#' \code{zmq.ctx.destroy()} will automatically be called for you.
#' 
#' \code{init.socket()} creates a ZeroMQ socket; serves as a high-level
#' binding for \code{zmq.socket()}, including handling freeing memory
#' automatically.  See also \code{ZMQ.ST()}.
#' 
#' \code{bind.socket()}:  see \code{zmq.bind()}.
#' 
#' \code{connect.socket()}:  see \code{zmq.connect()}
#' 
#' @param socket
#' A ZMQ socket.
#' @param data
#' An R object.
#' @param send.more
#' Logical; will more messages be sent?
#' @param serialize,unserialize
#' Logical; determines if serialize/unserialize should be called
#' on the sent/received data.
#' @param serialversion
#' NULL or numeric; the workspace format version to use when serializing.
#' NULL specifies the current default version. The only other supported
#' values are 2 and 3.
#' @param dont.wait
#' Logical; determines if reception is blocking.
#' @param context
#' A ZMQ context.
#' @param socket.type
#' The type of ZMQ socket as a string, of the form "ZMQ_type".  Valid 'type'
#' values are PAIR, PUB, SUB, REQ, REP, DEALER, PULL, PUSH, XPUB, XSUB, and
#' STERAM.
#' @param address
#' A valid address.  See details.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @keywords rzmq
#' @rdname xx_rzmq_wrapper
#' @name Wrapper Functions for rzmq
NULL



#' @rdname xx_rzmq_wrapper
#' @export
send.socket <- function(socket, data, send.more = FALSE, serialize = TRUE,
                        serialversion = NULL){
  if(send.more){
    flags <- ZMQ.SR()$SNDMORE
  } else{
    flags <- ZMQ.SR()$BLOCK
  }
  zmq.msg.send(data, socket, flags = flags, serialize = serialize,
               serialversion = serialversion)
}



#' @rdname xx_rzmq_wrapper
#' @export
receive.socket <- function(socket, unserialize = TRUE, dont.wait = FALSE){
  if(dont.wait){
    flags <- ZMQ.SR()$DONTWAIT
  } else{
    flags <- ZMQ.SR()$BLOCK
  }
  zmq.msg.recv(socket, flags = flags, unserialize = unserialize)
}



#' @rdname xx_rzmq_wrapper
#' @export
init.context <- function(){
  try.zmq.ctx.destroy <- function(ctx){
    invisible(suppressWarnings(zmq.ctx.destroy(ctx)))
  }
  
  ctx <- zmq.ctx.new()  
  reg.finalizer(ctx, try.zmq.ctx.destroy, onexit = TRUE)
  ctx 
}



#' @rdname xx_rzmq_wrapper
#' @export
init.socket <- function(context, socket.type){
  try.zmqt.close <- function(socket){
    invisible(suppressWarnings(zmq.close(socket)))
  }
  
  socket.type <- sub(".*_", "", socket.type)
  id <- which(names(ZMQ.ST()) == socket.type)
  if(length(id) != 1){
    stop("socket.type is not found.")
  } else{
    socket.type <- ZMQ.ST()[[id]]
  }
  
  socket <- zmq.socket(context, type = socket.type)
  reg.finalizer(socket, try.zmqt.close, onexit = TRUE)
  socket
}



#' @rdname xx_rzmq_wrapper
#' @export
bind.socket <- function(socket, address){
  zmq.bind(socket, address)
}



#' @rdname xx_rzmq_wrapper
#' @export
connect.socket <- function(socket, address){
  zmq.connect(socket, address)
}

#' File Transfer Functions
#' 
#' High level functions calling \code{zmq_send()} and \code{zmq_recv()}
#' to transfer a file in 200 KiB chunks.
#' 
#' @details
#' If no socket is passed, then by default \code{zmq.sendfile()} binds a
#' \code{ZMQ_PUSH} socket, and \code{zmq.recvfile()} connects to this with a
#' \code{ZMQ_PULL} socket. On the other hand, a PUSH/PULL, REQ/REP, or REP/REQ
#' socket pairing may be passed. In that case, the socket should already be
#' connected to the desired endpoint. Be careful not to pass the wrong socket
#' combination (e.g., do not do REQ/REQ), as this can put the processes in an
#' un-recoverable state.
#' 
#' @param port 
#' A valid tcp port.
#' @param endpoint
#' A ZMQ socket endpoint.
#' @param filename
#' The name (as a string) of the in/out files. The in and out file names
#' can be different.
#' @param verbose
#' Logical; determines if a progress bar should be shown.
#' @param flags
#' A flag for the method used by \code{zmq_sendfile} and
#' \code{zmq_recvfile}
#' @param forcebin
#' Force to read/send/recv/write in binary form. Typically for a Windows
#' system, text (ASCII) and binary files are processed differently.
#' If \code{TRUE}, "r+b" and "w+b" will be enforced in the C code.
#' This option is mainly for Windows.
#' @param ctx
#' A ZMQ ctx. If \code{NULL} (default), the function will initial one at
#' the beginning and destroy it after finishing file transfer.
#' @param socket
#' A ZMQ socket based on \code{ctx}.
#' If \code{NULL} (default), the function will create one at the beginning
#' and close it after finishing file transfer.
#' 
#' 
#' @return \code{zmq.sendfile()} and \code{zmq.recvfile()} return
#' number of bytes (invisible) in the sent message if successful,
#' otherwise returns -1 (invisible) and sets \code{errno} to the error
#' value, see ZeroMQ manual for details.
#' 
#' @author Drew Schmidt and Christian Heckendorf
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Run the sender and receiver code in separate R sessions.
#' 
#' # Receiver
#' library(pbdZMQ, quietly = TRUE)
#' zmq.recvfile(55555, "localhost", "/tmp/outfile", verbose=TRUE)
#' 
#' # Sender
#' library(pbdZMQ, quietly = TRUE)
#' zmq.sendfile(55555, "/tmp/infile", verbose=TRUE)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
#' @rdname b1_sendrecvfile
#' @name File Transfer Functions
NULL



# -----------------------------------------------------------------------------
# Send
# -----------------------------------------------------------------------------

#' @rdname b1_sendrecvfile
#' @export
zmq.sendfile <- function(port, filename, verbose=FALSE,
  flags = ZMQ.SR()$BLOCK, forcebin = FALSE, ctx = NULL, socket = NULL)
{
  if (is.null(socket))
  {
    if (is.null(ctx))
    {
      ctx <- zmq.ctx.new()
      ctx.destroy <- TRUE
    }
    else
      ctx.destroy <- FALSE
    
    socket <- zmq.socket(ctx, ZMQ.ST()$PUSH)
    socket.close <- TRUE
    
    endpoint <- address("*", port)
    zmq.bind(socket, endpoint)
  }
  else
  {
    ctx.destroy <- FALSE
    socket.close <- FALSE
  }
  
  type = attr(socket, "type")
  if (is.null(type))
    stop("unable to determine socket type")
  else if (type != ZMQ.ST()$PUSH && type != ZMQ.ST()$REQ && type != ZMQ.ST()$REP)
    stop("socket type must be one of PUSH, REQ, or REP (matching PULL, REP, and REQ respectively in zmq.recvfile())")
  
  fi <- file.info(filename)
  if (!is.na(fi$isdir) && !fi$isdir)
    filesize <- as.double(fi$size)
  else
    stop(paste("File does not exist:", filename)) 
  
  if (type == ZMQ.ST()$REP)
    receive.socket(socket)
  send.socket(socket, filesize)
  if (type == ZMQ.ST()$REQ)
    receive.socket(socket)
    
    ret <- .Call("R_zmq_send_file", socket, filename, as.integer(verbose),
      filesize, as.integer(flags), as.integer(forcebin), type, PACKAGE="pbdZMQ")
  
  if (socket.close || ctx.destroy)
    zmq.close(socket)
  if (ctx.destroy)
    zmq.ctx.destroy(ctx)
  
  invisible(ret)
}



# -----------------------------------------------------------------------------
# Receive
# -----------------------------------------------------------------------------

#' @rdname b1_sendrecvfile
#' @export
zmq.recvfile <- function(port, endpoint, filename, verbose=FALSE,
  flags = ZMQ.SR()$BLOCK, forcebin = FALSE, ctx = NULL, socket = NULL)
{
  if (is.null(socket))
  {
    if (is.null(ctx))
    {
      ctx <- zmq.ctx.new()
      ctx.destroy <- TRUE
    }
    else
      ctx.destroy <- FALSE
    
    socket <- zmq.socket(ctx, ZMQ.ST()$PULL)
    socket.close <- TRUE
    
    endpoint <- address(endpoint, port)
    zmq.connect(socket, endpoint)
  }
  else
  {
    ctx.destroy <- FALSE
    socket.close <- FALSE
  }
  
  type = attr(socket, "type")
  if (is.null(type))
    stop("unable to determine socket type")
  else if (type != ZMQ.ST()$PULL && type != ZMQ.ST()$REP && type != ZMQ.ST()$REQ)
    stop("socket type must be one of PULL, REP, or REQ (matching PUSH, REQ, and REP respectively in zmq.sendfile())")
  
  if (type == ZMQ.ST()$REQ)
    send.socket(socket, NULL)
  filesize <- receive.socket(socket)
  if (type == ZMQ.ST()$REP)
    send.socket(socket, NULL)
  
  ret <- .Call("R_zmq_recv_file", socket, filename, as.integer(verbose),
    filesize, as.integer(flags), as.integer(forcebin), type, PACKAGE="pbdZMQ")

  if (socket.close || ctx.destroy)
    zmq.close(socket)
  if (ctx.destroy)
    zmq.ctx.destroy(ctx)
  
  invisible(ret)
}

suppressPackageStartupMessages(library(pbdZMQ))

### rzmq interface

### In general and separate files
# server_context = init.context()
# server_socket = init.socket(server_context, "ZMQ_REP")
# client_context = init.context()
# client_socket = init.socket(client_context, "ZMQ_REQ")
### Server file
# bind.socket(server_socket, "tcp://*:55555")
### Client file
# connect.socket(client_socket, "tcp://localhost:55555")

### For CRAN testing in local (the same process) only to avoid block
cran_context = init.context()
server_socket = init.socket(cran_context, "ZMQ_REP")
client_socket = init.socket(cran_context, "ZMQ_REQ")
### Server
bind.socket(server_socket, "inproc://#1")
### Client
connect.socket(client_socket, "inproc://#1")


### Test rzmq
tester_rzmq = function(indata)
{
  send.socket(client_socket, indata)
  c2s <- receive.socket(server_socket)
  stopifnot(all.equal(c2s, indata))

  send.socket(server_socket, "ok")
  s2c <- receive.socket(client_socket)
  stopifnot(all.equal(s2c, "ok"))
}

tester_rzmq("test")
tester_rzmq(1:5)

#' Socket Functions
#' 
#' Socket functions
#' 
#' \code{zmq.socket()} initials a ZMQ socket given a ZMQ context \code{ctx} and
#' a socket \code{type}. See \code{\link{ZMQ.ST}()} for the possible values of
#' \code{type}. ZMQ defines several patterns for the socket type and utilize
#' them to communicate in different ways including request-reply,
#' publish-subscribe, pipeline, exclusive pair, and naive patterns.
#' 
#' \code{zmq.close()} destroys the ZMQ socket.
#' 
#' \code{zmq.bind()} binds the socket to a local endpoint and then accepts
#' incoming connections on that endpoint. See \code{endpoint} next for details.
#' 
#' \code{zmq.connect()} connects the socket to a remote endpoint and then
#' accepts outgoing connections on that endpoint. See \code{endpoint} next for
#' details.
#' 
#' \code{endpoint} is a string consisting of a transport :// followed by an
#' address. The transport specifies the underlying protocol to use. The address
#' specifies the transport-specific address to bind to.  pbdZMQ/ZMQ provides
#' the following transports: \tabular{ll}{ Transport \tab Usage \cr \code{tcp}
#' \tab unicast transport using TCP \cr \code{ipc} \tab local inter-process
#' communication transport \cr \code{inproc} \tab local in-process
#' (inter-thread) communication transport \cr \code{pgm,epgm} \tab reliable
#' multicast transport using PGM } *** warning: \code{epgm} is not turned on by
#' default in the pbdZMQ's internal ZeroMQ library. \cr *** warning: \code{ipc}
#' is not supported in Windows system.
#' 
#' \code{zmq.setsockopt()} is to set/change socket options.
#'
#' \code{zmq.getsockopt()} is to get socket options and returns
#' \code{option.value}.
#' 
#' @param ctx 
#' a ZMQ context
#' @param type 
#' a socket type
#' @param socket 
#' a ZMQ socket
#' @param endpoint 
#' a ZMQ socket endpoint
#' @param option.name 
#' an option name to the socket
#' @param option.value 
#' an option value to the option name
#' @param MC 
#' a message control, see \code{\link{ZMQ.MC}()} for details
#' 
#' @return \code{zmq.socket()} returns an R external pointer (\code{socket})
#' generated by ZMQ C API pointing to a socket if successful, otherwise returns
#' an R \code{NULL} and sets \code{errno} to the error value, see ZeroMQ manual
#' for details.
#' 
#' \code{zmq.close()} destroys the socket reference/pointer (\code{socket}) and
#' returns 0 if successful, otherwise returns -1 and sets \code{errno} to the
#' error value, see ZeroMQ manual for details.
#' 
#' \code{zmq.bind()} binds the socket to specific \code{endpoint} and returns 0
#' if successful, otherwise returns -1 and sets \code{errno} to the error
#' value, see ZeroMQ manual for details.
#' 
#' \code{zmq.connect()} connects the socket to specific \code{endpoint} and
#' returns 0 if successful, otherwise returns -1 and sets \code{errno} to the
#' error value, see ZeroMQ manual for details.
#' 
#' \code{zmq.setsockopt()} sets/changes the socket option and returns 0 if
#' successful, otherwise returns -1 and sets \code{errno} to the error value,
#' see ZeroMQ manual for details.
#'
#' \code{zmq.getsockopt()} returns the value of socket option,
#' see ZeroMQ manual for details.
#'
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Using request-reply pattern.
#' 
#' ### At the server, run next in background or the other windows.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' responder <- zmq.socket(context, ZMQ.ST()$REP)
#' zmq.bind(responder, "tcp://*:5555")
#' zmq.close(responder)
#' zmq.ctx.destroy(context)
#' 
#' 
#' ### At a client, run next in foreground.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' requester <- zmq.socket(context, ZMQ.ST()$REQ)
#' zmq.connect(requester, "tcp://localhost:5555")
#' zmq.close(requester)
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.ctx.new}()}, \code{\link{zmq.ctx.destroy}()}.
#' @rdname a1_socket
#' @name Socket Functions
NULL



#' @rdname a1_socket
#' @export
zmq.socket <- function(ctx, type = ZMQ.ST()$REP){
  ret <- .Call("R_zmq_socket", ctx, type, PACKAGE = "pbdZMQ")
  attr(ret, "type") = type
  ### Users are responsible to take care free and gc.
  # reg.finalizer(ret, zmq.close, TRUE)
  ret
}



#' @rdname a1_socket
#' @export
zmq.close <- function(socket){
  ret <- .Call("R_zmq_close", socket, PACKAGE = "pbdZMQ")
  invisible(ret)
}



#' @rdname a1_socket
#' @export
zmq.bind <- function(socket, endpoint, MC = ZMQ.MC()){
  ret <- .Call("R_zmq_bind", socket, endpoint, PACKAGE = "pbdZMQ")

  if(ret == -1){
    if(MC$stop.at.error){
      stop(paste("zmq.bind fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
    if(MC$warning.at.error){
      warning(paste("zmq.bind fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
  } else{
    return(invisible(ret))
  }
}



#' @rdname a1_socket
#' @export
zmq.connect <- function(socket, endpoint, MC = ZMQ.MC()){
  ret <- .Call("R_zmq_connect", socket, endpoint, PACKAGE = "pbdZMQ")

  if(ret == -1){
    if(MC$stop.at.error){
      stop(paste("zmq.connect fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
    if(MC$warning.at.error){
      warning(paste("zmq.connect fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
  } else{
    return(invisible(ret))
  }
}



#' @rdname a1_socket
#' @export
zmq.disconnect <- function(socket, endpoint, MC = ZMQ.MC()){
  ret <- .Call("R_zmq_disconnect", socket, endpoint, PACKAGE = "pbdZMQ")

  if(ret == -1){
    if(MC$stop.at.error){
      stop(paste("zmq.disconnect fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
    if(MC$warning.at.error){
      warning(paste("zmq.disconnect fails, ", endpoint, sep = ""))
      return(invisible(ret))
    }
  } else{
    return(invisible(ret))
  }
}



#' @rdname a1_socket
#' @export
zmq.setsockopt <- function(socket, option.name, option.value, MC = ZMQ.MC()){
  if(is.character(option.value)){
    option.type <- 0L
  } else if(is.integer(option.value)){
    option.type <- 1L
  } else{
    stop("Type of option.value is not implemented")
  }

  ret <- .Call("R_zmq_setsockopt", socket, option.name, option.value,
               option.type, PACKAGE = "pbdZMQ")

  if(ret == -1){
    if(MC$stop.at.error){
      stop(paste("zmq.setsockopt fails, ", option.value, sep = ""))
      return(invisible(ret))
    }
    if(MC$warning.at.error){
      warning(paste("zmq.setsockopt fails, ", option.value, sep = ""))
      return(invisible(ret))
    }
  } else{
    return(invisible(ret))
  }
}

#' @rdname a1_socket
#' @export
zmq.getsockopt <- function(socket, option.name, option.value, MC = ZMQ.MC()){
  if(is.character(option.value)){
    option.type <- 0L
  } else if(is.integer(option.value)){
    option.type <- 1L
  } else{
    stop("Type of option.value is not implemented")
  }

  ret <- .Call("R_zmq_getsockopt", socket, option.name, option.value,
               option.type, PACKAGE = "pbdZMQ")

  if(ret == -1){
    if(MC$stop.at.error){
      stop(paste("zmq.getsockopt fails, ", option.value, sep = ""))
      return(invisible(ret))
    }
    if(MC$warning.at.error){
      warning(paste("zmq.getsockopt fails, ", option.value, sep = ""))
      return(invisible(ret))
    }
  } else{
    return(invisible(option.value))
  }
}

.onLoad <- function(libname, pkgname){
  dn <- paste(libname, "/", pkgname, "/libs",
              Sys.getenv("R_ARCH"), "/", sep = "")
  i.file <- "libzmq.dll"
  fn <- paste(dn, i.file, sep = "")
  ### Load "libzmq.dll".
  test <- try(dyn.load(fn, local = FALSE), silent = TRUE)
  if(inherits(test, "try-error")){
    stop(paste("Could not load ", fn, sep = ""))
  }

  ### Load "pbdZMQ.dll".
  library.dynam("pbdZMQ", pkgname, libname)

  ### Preload to global environment.
  invisible(eval(parse(text = "pbdZMQ:::.zmqopt_init()")))

  invisible()
} # End of .onLoad().

.onUnload <- function(libpath){
  ### Unload "pbdZMQ.dll".
  library.dynam.unload("pbdZMQ", libpath)

  dn <- paste(libpath, "/libs",
              Sys.getenv("R_ARCH"), "/", sep = "")
  i.file <- "libzmq.dll"
  fn <- paste(dn, i.file, sep = "")
  if(file.exists(fn)){
    ### Unload "libzmq.dll".
    test <- try(dyn.unload(fn), silent = TRUE)
  }

  invisible()
} # End of .onUnload().

.onAttach <- function(libname, pkgname){
  invisible()
} # End of .onAttach().

#' A wrapper function for base::ls
#' 
#' @description
#' The \code{ls()} function with modification to avoid listing hidden
#' pbd objects.
#' 
#' @details
#' As the original \code{base::ls()}, it returns the names of the objects.
#'
#' @param name,pos,envir,all.names,pattern,sorted
#' as the original \code{base::ls()}.
#'
#' @return
#' As the original \code{base::ls()} except when \code{all.names} is \code{TRUE}
#' and \code{envir} is \code{.GlobalEnv}, hidden pbd objects such as
#' \code{.pbd_env} and \code{.pbdenv} will not be returned.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @rdname xx_ls_wrapper
#'
#' @examples
#' \dontrun{
#' library(pbdRPC, quietly = TRUE)
#' ls(all.names = TRUE)
#' base::ls(all.names = TRUE)
#' }
#'
#' @export
ls <- function(name, pos = -1L, envir = as.environment(pos), all.names = FALSE,
    pattern, sorted = TRUE)
{
  cmd <- "base::ls("
  if (!missing(name))
    cmd <- paste(cmd, as.character(name), ", ", sep = "")
  cmd <- paste(cmd, "all.names = ", all.names, ", ", sep = "")
  if (!missing(pattern))
    cmd <- paste(cmd, "pattern = \"", as.character(pattern), "\", ", sep = "")
  cmd <- paste(cmd, "sorted = ", sorted, ")", sep = "")

  ret <- eval(parse(text = cmd), envir = -2L)

  if (all.names == TRUE && environmentName(parent.frame()) == "R_GlobalEnv")
    ret <- grep("^\\.pbd(_|)env$", ret, value = TRUE, invert = TRUE)

  ret
}

### Do not delete this file and file name!!
### This file should be loaded before all other *.r files.

### This is to avoid the false positive messages from R CMD check.
###   "no visible binding for global variable"
### Suggested by Prof Brian Ripley
### ?globalVariables

if(getRversion() >= "2.15.1"){
  # utils::globalVariables(c(".pbd_env"))
}

#' Send Receive Functions
#' 
#' Send and receive functions
#' 
#' \code{zmq.send()} is a high level R function calling ZMQ C API
#' \code{zmq_send()} sending \code{buf} out.
#'
#' \code{zmq.recv()} is a high level R function calling ZMQ C API
#' \code{zmq_recv()} receiving buffers of length \code{len} according to the
#' \code{buf.type}.
#'
#' \code{flags} see \code{\link{ZMQ.SR}()} for detail options of send and
#' receive functions.
#' 
#' \code{buf.type} currently supports \code{char} and \code{raw} which are both
#' in R object format.
#' 
#' @param socket 
#' a ZMQ socket
#' @param buf 
#' a buffer to be sent
#' @param len 
#' a length of buffer to be received, default 1024 bytes
#' @param flags 
#' a flag for the method using by \code{zmq_send} and
#' \code{zmq_recv}
#' @param buf.type 
#' buffer type to be received
#' 
#' @return \code{zmq.send()} returns number of bytes (invisible) in the sent
#' message if successful, otherwise returns -1 (invisible) and sets
#' \code{errno} to the error value, see ZeroMQ manual for details.
#' 
#' \code{zmq.recv()} returns a list (\code{ret}) containing the received buffer
#' \code{ret$buf} and the length of received buffer (\code{ret$len} which is
#' less or equal to the input \code{len}) if successful, otherwise returns -1
#' and sets \code{errno} to the error value, see ZeroMQ manual for details.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Using request-reply pattern.
#' 
#' ### At the server, run next in background or the other window.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' responder <- zmq.socket(context, ZMQ.ST()$REP)
#' zmq.bind(responder, "tcp://*:5555")
#' for(i.res in 1:5){
#'   buf <- zmq.recv(responder, 10L)
#'   cat(buf$buf, "\n")
#'   Sys.sleep(0.5)
#'   zmq.send(responder, "World")
#' }
#' zmq.close(responder)
#' zmq.ctx.destroy(context)
#' 
#' 
#' ### At a client, run next in foreground.
#' library(pbdZMQ, quietly = TRUE)
#' 
#' context <- zmq.ctx.new()
#' requester <- zmq.socket(context, ZMQ.ST()$REQ)
#' zmq.connect(requester, "tcp://localhost:5555")
#' for(i.req in 1:5){
#'   cat("Sending Hello ", i.req, "\n")
#'   zmq.send(requester, "Hello")
#'   buf <- zmq.recv(requester, 10L)
#'   cat("Received World ", i.req, "\n")
#' }
#' zmq.close(requester)
#' zmq.ctx.destroy(context)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
#' @rdname b0_sendrecv
#' @name Send Receive Functions
NULL



#' @rdname b0_sendrecv
#' @export
zmq.send <- function(socket, buf, flags = ZMQ.SR()$BLOCK){
  if(is.character(buf)){
    ret <- zmq.send.char(socket, buf, nchar(buf), flags = flags)
  } else if(is.raw(buf)){
    ret <- zmq.send.raw(socket, buf, length(buf), flags = flags)
  } else{
    stop("buf type should be char or raw.")
  }
  invisible(ret)
}



zmq.send.char <- function(socket, buf, len, flags = ZMQ.SR()$BLOCK){
  ret <- .Call("R_zmq_send_char", socket, buf, as.integer(len),
               as.integer(flags), PACKAGE = "pbdZMQ")
  invisible(ret)
}



zmq.send.raw <- function(socket, buf, len, flags = ZMQ.SR()$BLOCK){
  ret <- .Call("R_zmq_send_raw", socket, buf, as.integer(len),
               as.integer(flags), PACKAGE = "pbdZMQ")
  invisible(ret)
}



#' @rdname b0_sendrecv
#' @export
zmq.recv <- function(socket, len = 1024L, flags = ZMQ.SR()$BLOCK,
    buf.type = c("char", "raw")){
  if(buf.type[1] == "char"){
    ret <- zmq.recv.char(socket, len, flags = flags)
  } else if(buf.type[1] == "raw"){
    ret <- zmq.recv.raw(socket, len, flags = flags)
  } else{
    stop("buf type should be char or raw.")
  }
  invisible(ret)
}



zmq.recv.char <- function(socket, len, flags = ZMQ.SR()$BLOCK){
  ret <- .Call("R_zmq_recv_char", socket, as.integer(len), as.integer(flags),
               PACKAGE = "pbdZMQ")
  invisible(ret)
}



zmq.recv.raw <- function(socket, len, flags = ZMQ.SR()$BLOCK){
  ret <- .Call("R_zmq_recv_raw", socket, as.integer(len), as.integer(flags),
               PACKAGE = "pbdZMQ")
  invisible(ret)
}

#' Sets of controls in pbdZMQ.
#' 
#' These sets of controls are used to provide default values in this package.
#' 
#' The elements of \code{.pbd_env$ZMQ.ST} are default values for socket types
#' as defined in `zmq.h' including \tabular{lcl}{ Elements \tab Value \tab
#' Usage \cr \code{PAIR} \tab 0L \tab socket type PAIR \cr \code{PUB} \tab 1L
#' \tab socket type PUB \cr \code{SUB} \tab 2L \tab socket type SUB \cr
#' \code{REQ} \tab 3L \tab socket type REQ \cr \code{REP} \tab 4L \tab socket
#' type REP \cr \code{DEALER} \tab 5L \tab socket type DEALER \cr \code{ROUTER}
#' \tab 6L \tab socket type ROUTER \cr \code{PULL} \tab 7L \tab socket type
#' PULL \cr \code{PUSH} \tab 8L \tab socket type PUSH \cr \code{XPUB} \tab 9L
#' \tab socket type XPUB \cr \code{XSUB} \tab 10L \tab socket type XSUB \cr
#' \code{STREAM} \tab 11L \tab socket type STREAM }
#' 
#' The elements of \code{.pbd_env$ZMQ.SO} are default values for socket
#' options as defined in `zmq.h' including 60 different values, see
#' \code{.pbd_env$ZMQ.SO} and `zmq.h' for details.
#' 
#' The elements of \code{.pbd_env$ZMQ.SR} are default values for send/recv
#' options as defined in `zmq.h' including \tabular{lcl}{ Elements \tab Value
#' \tab Usage \cr \code{BLOCK} \tab 0L \tab send/recv option BLOCK \cr
#' \code{DONTWAIT} \tab 1L \tab send/recv option DONTWAIT \cr \code{NOBLOCK}
#' \tab 1L \tab send/recv option NOBLOCK \cr \code{SNDMORE} \tab 2L \tab
#' send/recv option SNDMORE (not supported) }
#' 
#' The elements of \code{.pbd_env$ZMQ.MC} are default values for warning and
#' stop controls in R. These are not the ZeroMQ's internal default values. They
#' are defined as \tabular{lcl}{ Elements \tab Value \tab Usage \cr
#' \code{warning.at.error} \tab TRUE \tab if warn at error \cr
#' \code{stop.at.error} \tab TRUE \tab if stop at error }
#' 
#' @name ZMQ Control Environment
#' @aliases .pbd_env
#' @docType data
#' @format Objects contain several parameters for communicators and methods.
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' @keywords global variables
#' @seealso \code{\link{.zmqopt_init}()}.
#' @rdname a0_b_control
NULL

### These are fake. These only be here for reference and to fool
### ``R CMD check''.
### The real one ``in practicee and runtime'' is initialed by the
### .zmqopt_init() which is always called by .OnLoad() in "zzz.r" to avoid
### overloaded and can be really accessed by users instead of sealed by R
### after loaded.
###
### WCC: DO ``NOT'' remark ``NOR'' use the next.
# .pbd_env <- new.env()
# .pbd_env$ZMQ.MC <- ZMQ.MC()
# .pbd_env$ZMQ.SR <- ZMQ.SR()
# .pbd_env$ZMQ.SO <- ZMQ.SO()
# .pbd_env$ZMQ.ST <- ZMQ.ST()
# .pbd_env$ZMQ.PO <- ZMQ.PO()

suppressPackageStartupMessages(library(pbdZMQ))

addr = "notarealaddress.com.biz.google"
port = 8080

test = address(addr, port)
truth = paste0("tcp://", addr, ":", port)
stopifnot(identical(test, truth))

test = address(addr, transport="ipc")
truth = paste0("ipc://", addr)
stopifnot(identical(test, truth))

suppressPackageStartupMessages(library(pbdZMQ))

min = 49152
max = 65536

rp = random_port(min_port=min, max_port=max)
stopifnot(isTRUE(rp > min) && isTRUE(rp < max))

rop = random_open_port(min_port=49152, max_port=65536, max_tries=100)
stopifnot(isTRUE(rop > min) && isTRUE(rop < max))

#' Sets of controls in pbdZMQ.
#' 
#' These sets of controls are used to provide default values in this package.
#' 
#' @param warning.at.error,stop.at.error,check.eintr
#' Logical; if there is a messaging error, should there be an
#' R warning/error, or check user interrupt events.
#' @param POLLIN,POLLOUT,POLLERR
#' ZMQ poll options; see zmq.h for details.
#' @param BLOCK,DONTWAIT,NOBLOCK,SNDMORE
#' ZMQ socket options; see zmq.h for details.
#' @param AFFINITY,IDENTITY,SUBSCRIBE,UNSUBSCRIBE,RATE,RECOVERY_IVL,SNDBUF,RCVBUF,RCVMORE,FD,EVENTS,TYPE,LINGER,RECONNECT_IVL,BACKLOG,RECONNECT_IVL_MAX,MAXMSGSIZE,SNDHWM,RCVHWM,MULTICAST_HOPS,RCVTIMEO,SNDTIMEO,LAST_ENDPOINT,ROUTER_MANDATORY,TCP_KEEPALIVE,TCP_KEEPALIVE_CNT,TCP_KEEPALIVE_IDLE,TCP_KEEPALIVE_INTVL,TCP_ACCEPT_FILTER,IMMEDIATE,XPUB_VERBOSE,ROUTER_RAW,IPV6,MECHANISM,PLAIN_SERVER,PLAIN_USERNAME,PLAIN_PASSWORD,CURVE_SERVER,CURVE_PUBLICKEY,CURVE_SECRETKEY,CURVE_SERVERKEY,PROBE_ROUTER,REQ_CORRELATE,REQ_RELAXED,CONFLATE,ZAP_DOMAIN,ROUTER_HANDOVER,TOS,IPC_FILTER_PID,IPC_FILTER_UID,IPC_FILTER_GID,CONNECT_RID,GSSAPI_SERVER,GSSAPI_PRINCIPAL,GSSAPI_SERVICE_PRINCIPAL,GSSAPI_PLAINTEXT,HANDSHAKE_IVL,IDENTITY_FD,SOCKS_PROXY,XPUB_NODROP,BLOCKY,XPUB_MANUAL,XPUB_WELCOME_MSG,STREAM_NOTIFY,INVERT_MATCHING,HEARTBEAT_IVL,HEARTBEAT_TTL,HEARTBEAT_TIMEOUT,XPUB_VERBOSER,CONNECT_TIMEOUT,TCP_MAXRT,THREAD_SAFE,MULTICAST_MAXTPDU,VMCI_BUFFER_SIZE,VMCI_BUFFER_MIN_SIZE,VMCI_BUFFER_MAX_SIZE,VMCI_CONNECT_TIMEOUT,USE_FD,GSSAPI_PRINCIPAL_NAMETYPE,GSSAPI_SERVICE_PRINCIPAL_NAMETYPE,BINDTODEVICE,ZAP_ENFORCE_DOMAIN,LOOPBACK_FASTPATH,METADATA,MULTICAST_LOOP,ROUTER_NOTIFY,XPUB_MANUAL_LAST_VALUE,SOCKS_USERNAME,SOCKS_PASSWORD,IN_BATCH_SIZE,OUT_BATCH_SIZE,WSS_KEY_PEM,WSS_CERT_PEM,WSS_TRUST_PEM,WSS_HOSTNAME,WSS_TRUST_SYSTEM,ONLY_FIRST_SUBSCRIBE,RECONNECT_STOP,HELLO_MSG,DISCONNECT_MSG,PRIORITY
#' ZMQ socket options; see zmq.h for details.
#' @param PAIR,PUB,SUB,REQ,REP,DEALER,ROUTER,PULL,PUSH,XPUB,XSUB,STREAM
#' ZMQ socket types; see zmq.h for details.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @keywords global variables
#' @seealso \code{\link{.pbd_env}}.
#' @name ZMQ Control Functions
#' @rdname zz_zmq_control
NULL



### For ZMQ message control.
#' @export
#' @rdname zz_zmq_control
ZMQ.MC <- function(
  warning.at.error = TRUE,
  stop.at.error = FALSE,
  check.eintr = FALSE
){
  list(
    warning.at.error = warning.at.error,
    stop.at.error = stop.at.error,
    check.eintr = check.eintr
  )
}



### For ZMQ polll (I/O multiplexing).
#' @export
#' @rdname zz_zmq_control
ZMQ.PO <- function(
  POLLIN = 1L,
  POLLOUT = 2L,
  POLLERR = 4L# ,
  # POLLITEMS_DFLT = 16L
){
  list(
    POLLIN = POLLIN,
    POLLOUT = POLLOUT,
    POLLERR = POLLERR# ,
    # POLLITEMS_DFLT = POLLITEMS_DFLT
  )
}



### For ZMQ send/recv options, see zmq.h for details.
#' @export
#' @rdname zz_zmq_control
ZMQ.SR <- function(
  BLOCK = 0L,
  DONTWAIT = 1L,
  NOBLOCK = 1L,
  SNDMORE = 2L
){
  list(
    BLOCK = BLOCK,
    DONTWAIT = DONTWAIT,
    NOBLOCK = NOBLOCK,
    SNDMORE = SNDMORE
  )
}



### For ZMQ socket options, see zmq.h for details.
#' @export
#' @rdname zz_zmq_control
ZMQ.SO <- function(
  AFFINITY = 4L,
  IDENTITY = 5L,
  SUBSCRIBE = 6L,
  UNSUBSCRIBE = 7L,
  RATE = 8L,
  RECOVERY_IVL = 9L,
  SNDBUF = 11L,
  RCVBUF = 12L,
  RCVMORE = 13L,
  FD = 14L,
  EVENTS = 15L,
  TYPE = 16L,
  LINGER = 17L,
  RECONNECT_IVL = 18L,
  BACKLOG = 19L,
  RECONNECT_IVL_MAX = 21L,
  MAXMSGSIZE = 22L,
  SNDHWM = 23L,
  RCVHWM = 24L,
  MULTICAST_HOPS = 25L,
  RCVTIMEO = 27L,
  SNDTIMEO = 28L,
  LAST_ENDPOINT = 32L,
  ROUTER_MANDATORY = 33L,
  TCP_KEEPALIVE = 34L,
  TCP_KEEPALIVE_CNT = 35L,
  TCP_KEEPALIVE_IDLE = 36L,
  TCP_KEEPALIVE_INTVL = 37L,
  TCP_ACCEPT_FILTER = 38L,
  IMMEDIATE = 39L,
  XPUB_VERBOSE = 40L,
  ROUTER_RAW = 41L,
  IPV6 = 42L,
  MECHANISM = 43L,
  PLAIN_SERVER = 44L,
  PLAIN_USERNAME = 45L,
  PLAIN_PASSWORD = 46L,
  CURVE_SERVER = 47L,
  CURVE_PUBLICKEY = 48L,
  CURVE_SECRETKEY = 49L,
  CURVE_SERVERKEY = 50L,
  PROBE_ROUTER = 51L,
  REQ_CORRELATE = 52L,
  REQ_RELAXED = 53L,
  CONFLATE = 54L,
  ZAP_DOMAIN = 55L,
  ROUTER_HANDOVER = 56L,
  TOS = 57L,
  IPC_FILTER_PID = 58L,
  IPC_FILTER_UID = 59L,
  IPC_FILTER_GID = 60L,
  CONNECT_RID = 61L,
  GSSAPI_SERVER = 62L,
  GSSAPI_PRINCIPAL = 63L,
  GSSAPI_SERVICE_PRINCIPAL = 64L,
  GSSAPI_PLAINTEXT = 65L,
  HANDSHAKE_IVL = 66L,
  IDENTITY_FD = 67L,
  SOCKS_PROXY = 68L,
  XPUB_NODROP = 69L,
  BLOCKY = 70L,
  XPUB_MANUAL = 71L,
  XPUB_WELCOME_MSG = 72L,
  STREAM_NOTIFY = 73L,
  INVERT_MATCHING = 74L,
  HEARTBEAT_IVL = 75L,
  HEARTBEAT_TTL = 76L,
  HEARTBEAT_TIMEOUT = 77L,
  XPUB_VERBOSER = 78L,
  CONNECT_TIMEOUT = 79L,
  TCP_MAXRT = 80L,
  THREAD_SAFE = 81L,
  MULTICAST_MAXTPDU = 84L,
  VMCI_BUFFER_SIZE = 85L,
  VMCI_BUFFER_MIN_SIZE = 86L,
  VMCI_BUFFER_MAX_SIZE = 87L,
  VMCI_CONNECT_TIMEOUT = 88L,
  USE_FD = 89L,
  # Socket options after libzmq 4.2.2 (may not be stable).
  GSSAPI_PRINCIPAL_NAMETYPE = 90L,
  GSSAPI_SERVICE_PRINCIPAL_NAMETYPE = 91L,
  BINDTODEVICE = 92L,
  # DRAFT Socket options in libzmq 4.3.4 (may not be stable).
  # Need to define ZMQ_BUILD_DRAFT_API in C/C++ API. See zmq.h for details.
  ZAP_ENFORCE_DOMAIN = 93L,
  LOOPBACK_FASTPATH = 94L,
  METADATA = 95L,
  MULTICAST_LOOP = 96L,
  ROUTER_NOTIFY = 97L,
  XPUB_MANUAL_LAST_VALUE = 98L,
  SOCKS_USERNAME = 99L,
  SOCKS_PASSWORD = 100L,
  IN_BATCH_SIZE = 101L,
  OUT_BATCH_SIZE = 102L,
  WSS_KEY_PEM = 103L,
  WSS_CERT_PEM = 104L,
  WSS_TRUST_PEM = 105L,
  WSS_HOSTNAME = 106L,
  WSS_TRUST_SYSTEM = 107L,
  ONLY_FIRST_SUBSCRIBE = 108L,
  RECONNECT_STOP = 109L,
  HELLO_MSG = 110L,
  DISCONNECT_MSG = 111L,
  PRIORITY = 112L
){
  list(
    AFFINITY = AFFINITY,
    IDENTITY = IDENTITY,
    SUBSCRIBE = SUBSCRIBE,
    UNSUBSCRIBE = UNSUBSCRIBE,
    RATE = RATE,
    RECOVERY_IVL = RECOVERY_IVL,
    SNDBUF = SNDBUF,
    RCVBUF = RCVBUF,
    RCVMORE = RCVMORE,
    FD = FD,
    EVENTS = EVENTS,
    TYPE = TYPE,
    LINGER = LINGER,
    RECONNECT_IVL = RECONNECT_IVL,
    BACKLOG = BACKLOG,
    RECONNECT_IVL_MAX = RECONNECT_IVL_MAX,
    MAXMSGSIZE = MAXMSGSIZE,
    SNDHWM = SNDHWM,
    RCVHWM = RCVHWM,
    MULTICAST_HOPS = MULTICAST_HOPS,
    RCVTIMEO = RCVTIMEO,
    SNDTIMEO = SNDTIMEO,
    LAST_ENDPOINT = LAST_ENDPOINT,
    ROUTER_MANDATORY = ROUTER_MANDATORY,
    TCP_KEEPALIVE = TCP_KEEPALIVE,
    TCP_KEEPALIVE_CNT = TCP_KEEPALIVE_CNT,
    TCP_KEEPALIVE_IDLE = TCP_KEEPALIVE_IDLE,
    TCP_KEEPALIVE_INTVL = TCP_KEEPALIVE_INTVL,
    TCP_ACCEPT_FILTER = TCP_ACCEPT_FILTER,
    IMMEDIATE = IMMEDIATE,
    XPUB_VERBOSE = XPUB_VERBOSE,
    ROUTER_RAW = ROUTER_RAW,
    IPV6 = IPV6,
    MECHANISM = MECHANISM,
    PLAIN_SERVER = PLAIN_SERVER,
    PLAIN_USERNAME = PLAIN_USERNAME,
    PLAIN_PASSWORD = PLAIN_PASSWORD,
    CURVE_SERVER = CURVE_SERVER,
    CURVE_PUBLICKEY = CURVE_PUBLICKEY,
    CURVE_SECRETKEY = CURVE_SECRETKEY,
    CURVE_SERVERKEY = CURVE_SERVERKEY,
    PROBE_ROUTER = PROBE_ROUTER,
    REQ_CORRELATE = REQ_CORRELATE,
    REQ_RELAXED = REQ_RELAXED,
    CONFLATE = CONFLATE,
    ZAP_DOMAIN = ZAP_DOMAIN,
    ROUTER_HANDOVER = ROUTER_HANDOVER,
    TOS = TOS,
    IPC_FILTER_PID = IPC_FILTER_PID,
    IPC_FILTER_UID = IPC_FILTER_UID,
    IPC_FILTER_GID = IPC_FILTER_GID,
    CONNECT_RID = CONNECT_RID,
    GSSAPI_SERVER = GSSAPI_SERVER,
    GSSAPI_PRINCIPAL = GSSAPI_PRINCIPAL,
    GSSAPI_SERVICE_PRINCIPAL = GSSAPI_SERVICE_PRINCIPAL,
    GSSAPI_PLAINTEXT = GSSAPI_PLAINTEXT,
    HANDSHAKE_IVL = HANDSHAKE_IVL,
    IDENTITY_FD = IDENTITY_FD,
    SOCKS_PROXY = SOCKS_PROXY,
    XPUB_NODROP = XPUB_NODROP,
    BLOCKY = BLOCKY,
    XPUB_MANUAL = XPUB_MANUAL,
    XPUB_WELCOME_MSG = XPUB_WELCOME_MSG,
    STREAM_NOTIFY = STREAM_NOTIFY,
    INVERT_MATCHING = INVERT_MATCHING,
    HEARTBEAT_IVL = HEARTBEAT_IVL,
    HEARTBEAT_TTL = HEARTBEAT_TTL,
    HEARTBEAT_TIMEOUT = HEARTBEAT_TIMEOUT,
    XPUB_VERBOSER = XPUB_VERBOSER,
    CONNECT_TIMEOUT = CONNECT_TIMEOUT,
    TCP_MAXRT = TCP_MAXRT,
    THREAD_SAFE = THREAD_SAFE,
    MULTICAST_MAXTPDU = MULTICAST_MAXTPDU,
    VMCI_BUFFER_SIZE = VMCI_BUFFER_SIZE,
    VMCI_BUFFER_MIN_SIZE = VMCI_BUFFER_MIN_SIZE,
    VMCI_BUFFER_MAX_SIZE = VMCI_BUFFER_MAX_SIZE,
    VMCI_CONNECT_TIMEOUT = VMCI_CONNECT_TIMEOUT,
    USE_FD = USE_FD,
    GSSAPI_PRINCIPAL_NAMETYPE = GSSAPI_PRINCIPAL_NAMETYPE,
    GSSAPI_SERVICE_PRINCIPAL_NAMETYPE = GSSAPI_SERVICE_PRINCIPAL_NAMETYPE,
    BINDTODEVICE = BINDTODEVICE,
    ZAP_ENFORCE_DOMAIN = ZAP_ENFORCE_DOMAIN,
    LOOPBACK_FASTPATH = LOOPBACK_FASTPATH,
    METADATA = METADATA,
    MULTICAST_LOOP = MULTICAST_LOOP,
    ROUTER_NOTIFY = ROUTER_NOTIFY,
    XPUB_MANUAL_LAST_VALUE = XPUB_MANUAL_LAST_VALUE,
    SOCKS_USERNAME = SOCKS_USERNAME,
    SOCKS_PASSWORD = SOCKS_PASSWORD,
    IN_BATCH_SIZE = IN_BATCH_SIZE,
    OUT_BATCH_SIZE = OUT_BATCH_SIZE,
    WSS_KEY_PEM = WSS_KEY_PEM,
    WSS_CERT_PEM = WSS_CERT_PEM,
    WSS_TRUST_PEM = WSS_TRUST_PEM,
    WSS_HOSTNAME = WSS_HOSTNAME,
    WSS_TRUST_SYSTEM = WSS_TRUST_SYSTEM,
    ONLY_FIRST_SUBSCRIBE = ONLY_FIRST_SUBSCRIBE,
    RECONNECT_STOP = RECONNECT_STOP,
    HELLO_MSG = HELLO_MSG,
    DISCONNECT_MSG = DISCONNECT_MSG,
    PRIORITY = PRIORITY
  )
}



### For ZMQ socket types, see zmq.h for details.
#' @export
#' @rdname zz_zmq_control
ZMQ.ST <- function(
  PAIR = 0L,
  PUB = 1L,
  SUB = 2L,
  REQ = 3L,
  REP = 4L,
  DEALER = 5L,
  ROUTER = 6L,
  PULL = 7L,
  PUSH = 8L,
  XPUB = 9L,
  XSUB = 10L,
  STREAM = 11L
){
  list(
    PAIR = PAIR,
    PUB = PUB,
    SUB = SUB,
    REQ = REQ,
    REP = REP,
    DEALER = DEALER,
    ROUTER = ROUTER,
    PULL = PULL,
    PUSH = PUSH,
    XPUB = XPUB,
    XSUB = XSUB,
    STREAM = STREAM
  )
}

#' Set controls in pbdZMQ
#' 
#' Set control functions
#' 
#' \code{pbd_opt()} sets pbd options for ZMQ controls.
#'
#' \code{...} allows multiple options in
#' \code{envir$.pbd_env}, but only in a simple way.
#'
#' \code{bytext} allows to assign options by text in
#' \code{envir$.pbd_env}, but can assign advanced objects. For example,
#' \code{"option$suboption <- value"} will set
#' \code{envir$.pbd_env$option$suboption <- value}.
#' 
#' @param ... 
#' in argument format \code{option = value} to set
#' \code{.pbd_env$option <- value} inside the \code{envir}
#' @param bytext 
#' in text format \code{"option = value"} to set
#' \code{.pbd_env$option <- value} inside the \code{envir}.
#' @param envir 
#' by default the global environment is used.
#' 
#' @return 
#'  No value is returned.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com} and Drew Schmidt.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' library(pbdZMQ, quietly = TRUE)
#' 
#' ls(.pbd_env)
#' rm(.pbd_env)
#' .zmqopt_init()
#' ls(.pbd_env)
#'
#' .pbd_env$ZMQ.SR$BLOCK
#' pbd_opt(bytext = "ZMQ.SR$BLOCK <- 0L")
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{.pbd_env}}.
#' @rdname a0_b_pbd_opt
#' @name Set Control Functions

### Set pbd options.
#' @export
pbd_opt <- function(..., bytext = "", envir = .GlobalEnv){
  if(!exists(".pbd_env", envir = envir)){
    envir$.pbd_env <- new.env()
  } 

  arg <- list(...)
  if(length(arg) > 0){
    names.arg <- names(arg)
    if(is.null(names.arg) || any(names.arg == "")){
      stop("Options are all named.")
    }

    for(i.arg in 1:length(arg)){
      envir$.pbd_env[[names.arg[i.arg]]] <- arg[[i.arg]]
    }
  }

  if(bytext != ""){
    eval(parse(text = bytext), envir = envir$.pbd_env)
  }

  invisible()
} # End of pbd_opt().

suppressPackageStartupMessages(library(pbdZMQ))

### zmq interface

### In general and separate files
# server_context = zmq.ctx.new()
# server_socket = zmq.socket(server_context, ZMQ.ST()$REP)
# client_context = zmq.ctx.new()
# client_socket = zmq.socket(client_context, ZMQ.ST()$REQ)
### Server file
# zmq.setsockopt(server_socket, ZMQ.SO()$CONNECT_TIMEOUT, 1000L)
# zmq.setsockopt(server_socket, ZMQ.SO()$RCVTIMEO, 1000L)
# zmq.setsockopt(server_socket, ZMQ.SO()$SNDTIMEO, 1000L)
# zmq.bind(server_socket, "tcp://*:55555")
### client file
# zmq.setsockopt(client_socket, ZMQ.SO()$CONNECT_TIMEOUT, 1000L)
# zmq.setsockopt(client_socket, ZMQ.SO()$RCVTIMEO, 1000L)
# zmq.setsockopt(client_socket, ZMQ.SO()$SNDTIMEO, 1000L)
# zmq.connect(client_socket, "tcp://localhost:55555")

### For CRAN testing in local (the same process) only to avoid block
cran_context = zmq.ctx.new()
server_socket = zmq.socket(cran_context, ZMQ.ST()$REP)
client_socket = zmq.socket(cran_context, ZMQ.ST()$REQ)
### Server
zmq.bind(server_socket, "inproc://#1")
### client
zmq.connect(client_socket, "inproc://#1")


### Test pbdZMQ
tester = function(indata)
{
  zmq.msg.send(indata, client_socket)
  c2s <- zmq.msg.recv(server_socket)
  stopifnot(all.equal(c2s, indata))

  zmq.send(server_socket, "ok")
  s2c <- zmq.recv(client_socket)
  stopifnot(all.equal(s2c$buf, "ok"))
}

tester("test")
tester(1:5)

zmq.close(server_socket)
zmq.close(client_socket)

#' Transfer Functions for Files or Directories
#' 
#' High level functions calling \code{zmq.sendfile()} and \code{zmq.recvfile()}
#' to zip, transfer, and unzip small files or directories contains small files.
#' 
#' @details
#' \code{zmq.senddir()} calls \code{zmq.senddir()}, and
#' \code{zmq.recvdir()} calls \code{zmq.recvdir()}.
#' 
#' @param port 
#' A valid tcp port to be passed to \code{zmq.sendfile()} and
#' \code{zmq.recvfile()}.
#' @param endpoint
#' A ZMQ socket endpoint to be passed to \code{zmq.sendfile()} and
#' \code{zmq.recvfile()}.
#' @param infiles
#' The name (as a string) vector of the in files to be zipped and to be
#' sent away.
#' @param outfile
#' The name (as a string) of the out file to be saved on the disk.
#' If \code{outfile = NULL} and \code{exdir = NULL}, a tempfile will be
#' used and the tempfile nanme will be returned.
#' @param exdir
#' The name (as a string) of the out directory to save the unzip files
#' unzipped from the received \code{outfile}.
#' @param verbose
#' Logical; determines if a progress bar should be shown.
#' @param flags
#' A flag for the method used by \code{zmq_sendfile} and
#' \code{zmq_recvfile}
#' @param ctx
#' A ZMQ ctx. If \code{NULL} (default), the function will initial one at
#' the beginning and destroy it after finishing file transfer.
#' @param socket
#' A ZMQ socket based on \code{ctx}.
#' If \code{NULL} (default), the function will create one at the beginning
#' and close it after finishing file transfer.
#' 
#' 
#' @return \code{zmq.senddir()} and \code{zmq.recvdir()} return
#' number of bytes (invisible) in the sent message if successful,
#' otherwise returns -1 (invisible) and sets \code{errno} to the error
#' value, see ZeroMQ manual for details.
#' In addition, \code{zmq.recvdir()} returns a zipped file name in a list.
#' 
#' @author Wei-Chen Chen
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' ### Run the sender and receiver code in separate R sessions.
#' 
#' ### Receiver
#' library(pbdZMQ, quietly = TRUE)
#' zmq.recvdir(55555, "localhost", outfile = "./backup_2019.zip",
#'             verbose = TRUE)
#' ### or unzip to exdir
#' # zmq.recvdir(55555, "localhost", exdir = "./backup_2019", verbose = TRUE)
#' 
#' ### Sender
#' library(pbdZMQ, quietly = TRUE)
#' zmq.senddir(55555, c("./pbdZMQ/R", "./pbdZMQ/src"), verbose = TRUE)
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{zmq.sendfile}()}, \code{\link{zmq.recvfile}()}.
#' @rdname b2_sendrecvdir
#' @name Transfer Functions for Files or Directories
#' @importFrom utils unzip zip
#' 
NULL



#' @rdname b2_sendrecvdir
#' @export
zmq.senddir <- function(port, infiles, verbose = FALSE,
                        flags = ZMQ.SR()$BLOCK,
                        ctx = NULL, socket = NULL)
{
  if (is.null(socket))
  {
    if (is.null(ctx))
    { 
      ctx <- zmq.ctx.new()
      ctx.destroy <- TRUE
    }
    else
      ctx.destroy <- FALSE

    socket <- zmq.socket(ctx, ZMQ.ST()$PUSH)
    socket.close <- TRUE
    
    endpoint <- address("*", port)
    zmq.bind(socket, endpoint)
  }
  else
  {
    ctx.destroy <- FALSE
    socket.close <- FALSE
  }

  type = attr(socket, "type")
  if (is.null(type))
    stop("unable to determine socket type")
  else if (type != ZMQ.ST()$PUSH && type != ZMQ.ST()$REQ && type != ZMQ.ST()$REP)
    stop("socket type must be one of PUSH, REQ, or REP (matching PULL, REP, and REQ respectively in zmq.recvfile())")

  if (!verbose)
    extras <- "-q"
  else
    extras <- ""

  tmp.fn <- tempfile(fileext = ".zip")
  zip(tmp.fn, infiles, extras = extras)
  ret <- zmq.sendfile(port, tmp.fn, verbose = verbose, flags = flags,
                      forcebin = TRUE, ctx = ctx, socket = socket)
  
  if (socket.close || ctx.destroy)
    zmq.close(socket)

  if (ctx.destroy)
    zmq.ctx.destroy(ctx)
  
  invisible(list(ret = ret, infile = tmp.fn))
}


#' @rdname b2_sendrecvdir
#' @export
zmq.recvdir <- function(port, endpoint, outfile = NULL, exdir = NULL,
                        verbose = FALSE, flags = ZMQ.SR()$BLOCK,
                        ctx = NULL, socket = NULL)
{
  if (is.null(socket))
  {
    if (is.null(ctx))
    {
      ctx <- zmq.ctx.new()
      ctx.destroy <- TRUE
    }
    else
      ctx.destroy <- FALSE

    socket <- zmq.socket(ctx, ZMQ.ST()$PULL)
    socket.close <- TRUE

    endpoint <- address(endpoint, port)
    zmq.connect(socket, endpoint)
  }
  else
  {
    ctx.destroy <- FALSE
    socket.close <- FALSE
  }

  type = attr(socket, "type")
  if (is.null(type))
    stop("unable to determine socket type")
  else if (type != ZMQ.ST()$PULL && type != ZMQ.ST()$REP && type != ZMQ.ST()$REQ)
    stop("socket type must be one of PULL, REP, or REQ (matching PUSH, REQ, and REP respectively in zmq.sendfile())")

  if (is.null(outfile))
    outfile <- tempfile()

  ret <- zmq.recvfile(port, endpoint, outfile, verbose = verbose,
                      flags = flags, forcebin = TRUE,
                      ctx = ctx, socket = socket)

  if (!is.null(exdir))
    unzip(outfile, exdir = exdir)

  if (socket.close || ctx.destroy)
    zmq.close(socket)

  if (ctx.destroy)
    zmq.ctx.destroy(ctx)
  
  invisible(list(ret = ret, outfile = outfile))
}

#' Initial controls in pbdZMQ
#' 
#' Initial control functions
#' 
#' \code{.zmqopt_init()} initials default ZMQ controls.
#' \code{.zmqopt_get()} gets a ZMQ control.
#' \code{.zmqopt_set()} sets a ZMQ control.
#' 
#' @param envir 
#' an environment where ZMQ controls locate
#' @param val
#' a value to be set
#' @param main
#' a variable to be get from or set to
#' @param sub
#' a subvariable to be get from or set to
#' 
#' @return 
#' \code{.zmqopt_init()} initial the ZMQ control
#' at \code{envir}.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' library(pbdZMQ, quietly = TRUE)
#' 
#' ls(.pbd_env)
#' rm(.pbd_env)
#' .zmqopt_init()
#' ls(.pbd_env)
#'
#' .pbd_env$ZMQ.SR$BLOCK
#' pbd_opt(bytext = "ZMQ.SR$BLOCK = 0L")
#' }
#' 
#' @keywords programming
#' @seealso \code{\link{.pbd_env}}.
#' @rdname a0_c_options
#' @name Initial Control Functions


### Get ZMQ options.
#' @export
.zmqopt_get <- function(main, sub = NULL, envir = .GlobalEnv){
  if(!is.null(sub)){
    envir$.pbd_env[[main]][[sub]]
  } else{
    envir$.pbd_env[[main]]
  }
} # End of .zmqopt_get().

### Set ZMQ options.
#' @export
#' @rdname a0_c_options
.zmqopt_set <- function(val, main, sub = NULL, envir = .GlobalEnv){
  if(!is.null(sub)){
    envir$.pbd_env[[main]][[sub]] <- val
  } else{
    envir$.pbd_env[[main]] <- val
  }
  invisible()
} # End of .zmqopt_set().

### Initial ZMQ options.
#' @export
#' @rdname a0_c_options
.zmqopt_init <- function(envir = .GlobalEnv){
  if(!exists(".pbd_env", envir = envir)){
    envir$.pbd_env <- new.env()
  } 
  envir$.pbd_env$ZMQ.MC <- ZMQ.MC()
  envir$.pbd_env$ZMQ.SR <- ZMQ.SR()
  envir$.pbd_env$ZMQ.SO <- ZMQ.SO()
  envir$.pbd_env$ZMQ.ST <- ZMQ.ST()
  envir$.pbd_env$ZMQ.PO <- ZMQ.PO()

  invisible()
} # End of .zmqopt_init().

#' ZMQ Flags
#' 
#' ZMQ Flags
#' 
#' \code{get.zmq.cppflags()} gets CFLAGS or CPPFLAGS
#'
#' \code{get.zmq.ldflags()} gets LDFLAGS for libzmq.so, libzmq.dll, or libzmq.*.dylib
#'
#' \code{get.pbdZMQ.ldflags()} gets LDFLAGS for pbdZMQ.so or pbdZMQ.dll
#' 
#' \code{test.load.zmq()} tests load libzmq and pbdZMQ shared libraries
#' 
#' @param arch 
#' '' (default) for non-windows or '/i386' and '/ix64' for windows
#' @param package
#' the pbdZMQ package 
#' 
#' @return flags to compile and link with ZMQ.
#' 
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @examples
#' \dontrun{
#' get.zmq.cppflags(arch = '/i386')
#' get.zmq.ldflags(arch = '/x64')
#' get.pbdZMQ.ldflags(arch = '/x64')
#' test.load.zmq(arch = '/x64')
#' }
#' 
#' @keywords compile
#' @rdname zz_zmq_flags
#' @name ZMQ Flags
NULL



#' @rdname zz_zmq_flags
#' @export
get.zmq.ldflags <- function(arch = '', package = "pbdZMQ"){
  if(arch == "/i386" || arch == "/x64"){
    file.name <- paste("./libs", arch, "/", sep = "")
    dir.path <- tools::file_path_as_absolute(
                  system.file(file.name, package = package))
    zmq.ldflags <- paste("-L\"", dir.path, "\" -lzmq", sep = "")
  } else{
    ### For non windows system.
    file.name <- paste("./etc", arch, "/Makeconf", sep = "")
    file.path <- tools::file_path_as_absolute(
                   system.file(file.name, package = package))
    ret <- scan(file.path, what = character(), sep = "\n", quiet = TRUE)

    ### Check if system zmq is used.
    arg <- "SYSTEM_ZMQ_LIBDIR"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    sys.zmq.ld <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check if external zmq is used.
    arg <- "EXTERNAL_ZMQ_LDFLAGS"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    ext.zmq.ld <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check if internal zmq is used.
    arg <- "ENABLE_INTERNAL_ZMQ"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    en.int.zmq <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check which zmq should be used.
    if((sys.zmq.ld == "" && ext.zmq.ld == "") || en.int.zmq == "yes"){
      file.name <- paste("./libs", arch, "/", sep = "")
      dir.path <- tools::file_path_as_absolute(
                    system.file(file.name, package = package))
      if(Sys.info()[['sysname']] == "Darwin"){
        lib.osx <- list.files(dir.path, pattern = "libzmq\\.(.*)\\.dylib")
        i.ver <- gsub("libzmq\\.(.*)\\.dylib", "\\1", lib.osx)
	i.ver <- max(as.integer(i.ver))
        zmq.ldflags <- paste("-L\"", dir.path, "\" -lzmq.", i.ver, sep = "")
      } else{
        zmq.ldflags <- paste("-L\"", dir.path, "\" -lzmq", sep = "")
      }
    } else{
      arg <- "ZMQ_LDFLAGS"
      id <- grep(paste("^", arg, " = ", sep = ""), ret)
      zmq.ldflags <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1",
                          ret[id[1]])
    }
  }

  ### Cat back to "Makevars".
  cat(zmq.ldflags)

  invisible(zmq.ldflags)
} # End of get.zmq.ldflags().



#' @rdname zz_zmq_flags
#' @export
get.zmq.cppflags <- function(arch = '', package = "pbdZMQ"){
  if(arch == "/i386" || arch == "/x64"){
    file.name <- paste("./zmq", arch, "/include", sep = "")
    dir.path <- tools::file_path_as_absolute(
                  system.file(file.name, package = package))
    zmq.cppflags <- paste("-I\"", dir.path, "\"", sep = "")
  } else{
    ### For non windows system.
    file.name <- paste("./etc", arch, "/Makeconf", sep = "")
    file.path <- tools::file_path_as_absolute(
                   system.file(file.name, package = package))
    ret <- scan(file.path, what = character(), sep = "\n", quiet = TRUE)

    ### Check if system zmq is used.
    arg <- "SYSTEM_ZMQ_INCLUDEDIR"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    sys.zmq.inc <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check if external zmq is used.
    arg <- "EXTERNAL_ZMQ_INCLUDE"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    ext.zmq.inc <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check if internal zmq is used.
    arg <- "ENABLE_INTERNAL_ZMQ"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    en.int.zmq <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1", ret[id[1]])

    ### Check which zmq should be used.
    if((sys.zmq.inc == "" && ext.zmq.inc == "") || en.int.zmq == "yes"){
      file.name <- paste("./zmq", arch, "/include/", sep = "")
      dir.path <- tools::file_path_as_absolute(
                    system.file(file.name, package = package))
      zmq.cppflags <- paste("-I\"", dir.path, "\"", sep = "")
    } else{
      arg <- "ZMQ_INCLUDE"
      id <- grep(paste("^", arg, " = ", sep = ""), ret)
      zmq.cppflags <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1",
                           ret[id[1]])
    }

    ### Check if libzmq>=4.3.0 is used.
    arg <- "GET_SYSTEM_ZMQ_430"
    id <- grep(paste("^", arg, " = ", sep = ""), ret)
    sys.zmq.430 <- gsub(paste("^", arg, " = (.*)", sep = ""), "\\1",
                        ret[id[1]])

    chk.zmq.h.430 <- "no"
    if(sys.zmq.430 == "no"){
      ### Check with zmq.h (assuming one only "-I..." in zmq.cppflags)
      path.zmq.h <- gsub("^-I(.*)$", "\\1", zmq.cppflags)
      path.zmq.h <- gsub("^\\\"(.*)\\\"$", "\\1", path.zmq.h)
      f.zmq.h <- paste(path.zmq.h, "/zmq.h", sep = "")
      ret.zmq.h <- scan(f.zmq.h, what = character(), sep = "\n", quiet = TRUE)

      id.major <- grep("^#define ZMQ_VERSION_MAJOR ", ret.zmq.h)
      id.minor <- grep("^#define ZMQ_VERSION_MINOR ", ret.zmq.h)
      v.major <- gsub("^#define ZMQ_VERSION_MAJOR (.*)$", "\\1",
                      ret.zmq.h[id.major])
      v.minor <- gsub("^#define ZMQ_VERSION_MINOR (.*)$", "\\1",
                      ret.zmq.h[id.minor])
      if(v.major >=4 && v.minor >= 3){
        chk.zmq.h.430 <- "yes"
      }
    }

    if((sys.zmq.430 == "yes" && en.int.zmq != "yes") || chk.zmq.h.430 == "yes"){
      zmq.cppflags <- paste(zmq.cppflags, " -DENABLE_DRAFTS=ON", sep = "")
    }
  }

  ### Cat back to "Makevars".
  cat(zmq.cppflags)

  invisible(zmq.cppflags)
} # End of get.zmq.cppflags().



#' @rdname zz_zmq_flags
#' @export
test.load.zmq <- function(arch = '', package = "pbdZMQ"){
  file.name <- paste("./libs", arch, "/", sep = "")
  dir.path <- tools::file_path_as_absolute(
                system.file(file.name, package = package))

  files <- c("libzmq.so", "libzmq.so.dSYM", "libzmq.dylib", "libzmq.4.dylib",
             "libzmq.5.dylib", "libzmq.dll")
  for(i.file in files){
    fn <- paste(dir.path, "/", i.file, sep = "")
    if(file.exists(fn)){
      ret <- try(dyn.load(fn, local = FALSE), silent = TRUE)
      print(ret)
      cat("\n")
    }
  }

  invisible(NULL)
} # End of test.load.zmq().



#' @rdname zz_zmq_flags
#' @export
get.pbdZMQ.ldflags <- function(arch = '', package = "pbdZMQ"){
  file.name <- paste("./libs", arch, "/", sep = "")
  dir.path <- tools::file_path_as_absolute(
                system.file(file.name, package = package))

  if(arch == "/i386" || arch == "/x64"){
    pbdZMQ.ldflags <- paste(dir.path, "/pbdZMQ.dll", sep = "")
  } else{
    pbdZMQ.ldflags <- paste(dir.path, "/pbdZMQ.so", sep = "")
  }

  ### Cat back to "Makevars".
  cat(pbdZMQ.ldflags)

  invisible(pbdZMQ.ldflags)
} # End of get.pbdZMQ.ldflags().

#' Programming with Big Data -- Interface to ZeroMQ
#' 
#' ZeroMQ is a well-known library for high-performance
#' asynchronous messaging in scalable, distributed applications.  This
#' package provides high level R wrapper functions to easily utilize
#' ZeroMQ. We mainly focus on interactive client/server programming 
#' frameworks. For convenience, a minimal ZeroMQ library (4.1.0 rc1)
#' is shipped with pbdZMQ, which can be used if no system installation
#' of ZeroMQ is available.  A few wrapper functions compatible with
#' rzmq are also provided.
#' 
#' The install command using default \pkg{pbdZMQ}'s internal ZeroMQ library is
#' \cr \cr 
#' \code{> R CMD INSTALL pbdZMQ_0.1-0.tar.gz} \cr 
#' \code{--configure-args="--enable-internal-zmq"} 
#' \cr \cr 
#' Other available variables include 
#' \tabular{ll}{
#'   Variable \tab Default \cr 
#'   \code{ZMQ_INCLUDE} \tab \code{-I./zmqsrc/include} \cr 
#'   \code{ZMQ_LDFLAGS} \tab \code{-L./ -lzmq} \cr
#'   \code{ZMQ_POLLER} \tab \code{select} \cr
#' } 
#' See the package source file \code{pbdZMQ/configure.ac} for details.
#' 
#' For installation using an external ZeroMQ library, see the package source
#' file \code{pbdZMQ/INSTALL} for details.
#' 
#' @name pbdZMQ-package
#' @aliases pbdZMQ-package pbdZMQ
#' @docType package
#' @author Wei-Chen Chen \email{wccsnow@@gmail.com}.
#' @seealso \code{\link{zmq.ctx.new}()}, \code{\link{zmq.socket}()}.
#' 
#' @references ZeroMQ/4.1.0 API Reference:
#' \url{http://api.zeromq.org/4-1:_start}
#' 
#' Programming with Big Data in R Website: \url{https://pbdr.org/}
#' 
#' @keywords package
#' 
#' @rdname a0_a_pbdZMQ-package
#'
NULL

#!/bin/sh

R_HOME=`Rscript -e 'cat(R.home()[1])'`
JSS_BST=${R_HOME}/share/texmf/bibtex/bst/jss.bst
JSS_CLS=${R_HOME}/share/texmf/tex/latex/jss.cls

rm *.aux *.bbl *.blg *.log *.out *.toc *.bst *.cls
cp ${JSS_BST} ./
cp ${JSS_CLS} ./

pdflatex pbdZMQ-guide.Rnw
bibtex pbdZMQ-guide
pdflatex pbdZMQ-guide.Rnw
pdflatex pbdZMQ-guide.Rnw
pdflatex pbdZMQ-guide.Rnw

rm *.aux *.bbl *.blg *.log *.out *.toc *.bst *.cls
mv -f *.pdf ../inst/doc/
cp -f *.Rnw ../inst/doc/

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/222_export_opt.r
\name{Initial Control Functions}
\alias{Initial Control Functions}
\alias{.zmqopt_get}
\alias{.zmqopt_set}
\alias{.zmqopt_init}
\title{Initial controls in pbdZMQ}
\usage{
.zmqopt_get(main, sub = NULL, envir = .GlobalEnv)

.zmqopt_set(val, main, sub = NULL, envir = .GlobalEnv)

.zmqopt_init(envir = .GlobalEnv)
}
\arguments{
\item{main}{a variable to be get from or set to}

\item{sub}{a subvariable to be get from or set to}

\item{envir}{an environment where ZMQ controls locate}

\item{val}{a value to be set}
}
\value{
\code{.zmqopt_init()} initial the ZMQ control
at \code{envir}.
}
\description{
Initial control functions
}
\details{
\code{.zmqopt_init()} initials default ZMQ controls.
\code{.zmqopt_get()} gets a ZMQ control.
\code{.zmqopt_set()} sets a ZMQ control.
}
\examples{
\dontrun{
library(pbdZMQ, quietly = TRUE)

ls(.pbd_env)
rm(.pbd_env)
.zmqopt_init()
ls(.pbd_env)

.pbd_env$ZMQ.SR$BLOCK
pbd_opt(bytext = "ZMQ.SR$BLOCK = 0L")
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{.pbd_env}}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overwrite_shpkg.r
\name{Overwrite shpkg}
\alias{Overwrite shpkg}
\alias{overwrite.shpkg.rpath}
\title{Overwrite rpath of linked shared library in osx}
\usage{
overwrite.shpkg.rpath(
  mylib = NULL,
  mypkg = "JuniperKernel",
  linkingto = "pbdZMQ",
  shlib = "zmq"
)
}
\arguments{
\item{mylib}{the path where \code{mypkg} was installed (default \code{NULL} that will
search from R's path)}

\item{mypkg}{the package for where \code{mypkg.so} will be checked or updated}

\item{linkingto}{the package for where \code{libshpkg*.dylib} is located}

\item{shlib}{name of shlib to be searched for}
}
\description{
Overwrite rpath of linked shared library
(e.g. \code{JuniperKernel/libs/JuniperKernel.so}
in osx only.
Typically, it is called by \code{.onLoad()} to update rpath if
\code{pbdZMQ} or \code{pbdZMQ/libs/libzmq.*.dylib} was moved to
a personal directory
(e.g. the binary package was installed to a none default path).
The commands \code{otool} and \code{install_name_tool} are required.
Permission may be needed (e.g. \code{sudo}) to overwrite the shared
library.
}
\examples{
\dontrun{
### Called by .onLoad() within "JuniperKernel/R/zzz.R"
overwrite.shpkg.rpath(mypkg = "JuniperKernel",
                      linkingto = "pbdZMQ",
                      shlib = "zmq")
}

}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\keyword{compile}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rzmq_wrapper.r
\name{Wrapper Functions for rzmq}
\alias{Wrapper Functions for rzmq}
\alias{send.socket}
\alias{receive.socket}
\alias{init.context}
\alias{init.socket}
\alias{bind.socket}
\alias{connect.socket}
\title{All Wrapper Functions for rzmq}
\usage{
send.socket(
  socket,
  data,
  send.more = FALSE,
  serialize = TRUE,
  serialversion = NULL
)

receive.socket(socket, unserialize = TRUE, dont.wait = FALSE)

init.context()

init.socket(context, socket.type)

bind.socket(socket, address)

connect.socket(socket, address)
}
\arguments{
\item{socket}{A ZMQ socket.}

\item{data}{An R object.}

\item{send.more}{Logical; will more messages be sent?}

\item{serialize, unserialize}{Logical; determines if serialize/unserialize should be called
on the sent/received data.}

\item{serialversion}{NULL or numeric; the workspace format version to use when serializing.
NULL specifies the current default version. The only other supported
values are 2 and 3.}

\item{dont.wait}{Logical; determines if reception is blocking.}

\item{context}{A ZMQ context.}

\item{socket.type}{The type of ZMQ socket as a string, of the form "ZMQ_type".  Valid 'type'
values are PAIR, PUB, SUB, REQ, REP, DEALER, PULL, PUSH, XPUB, XSUB, and
STERAM.}

\item{address}{A valid address.  See details.}
}
\description{
Wrapper functions for backwards compatibility with rzmq.  See vignette
for examples.
}
\details{
\code{send.socket()}/\code{receive.socket()} send/receive messages over
a socket.  These are simple wrappers around \code{zmq.msg.send()} and
\code{zmq.msg.receive()}, respectively.

\code{init.context()} creates a new ZeroMQ context.  A useful wrapper
around \code{zmq.ctx.new()} which handles freeing memory for you, i.e.
\code{zmq.ctx.destroy()} will automatically be called for you.

\code{init.socket()} creates a ZeroMQ socket; serves as a high-level
binding for \code{zmq.socket()}, including handling freeing memory
automatically.  See also \code{ZMQ.ST()}.

\code{bind.socket()}:  see \code{zmq.bind()}.

\code{connect.socket()}:  see \code{zmq.connect()}
}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{rzmq}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_sendrecv.r
\name{Send Receive Functions}
\alias{Send Receive Functions}
\alias{zmq.send}
\alias{zmq.recv}
\title{Send Receive Functions}
\usage{
zmq.send(socket, buf, flags = ZMQ.SR()$BLOCK)

zmq.recv(
  socket,
  len = 1024L,
  flags = ZMQ.SR()$BLOCK,
  buf.type = c("char", "raw")
)
}
\arguments{
\item{socket}{a ZMQ socket}

\item{buf}{a buffer to be sent}

\item{flags}{a flag for the method using by \code{zmq_send} and
\code{zmq_recv}}

\item{len}{a length of buffer to be received, default 1024 bytes}

\item{buf.type}{buffer type to be received}
}
\value{
\code{zmq.send()} returns number of bytes (invisible) in the sent
message if successful, otherwise returns -1 (invisible) and sets
\code{errno} to the error value, see ZeroMQ manual for details.

\code{zmq.recv()} returns a list (\code{ret}) containing the received buffer
\code{ret$buf} and the length of received buffer (\code{ret$len} which is
less or equal to the input \code{len}) if successful, otherwise returns -1
and sets \code{errno} to the error value, see ZeroMQ manual for details.
}
\description{
Send and receive functions
}
\details{
\code{zmq.send()} is a high level R function calling ZMQ C API
\code{zmq_send()} sending \code{buf} out.

\code{zmq.recv()} is a high level R function calling ZMQ C API
\code{zmq_recv()} receiving buffers of length \code{len} according to the
\code{buf.type}.

\code{flags} see \code{\link{ZMQ.SR}()} for detail options of send and
receive functions.

\code{buf.type} currently supports \code{char} and \code{raw} which are both
in R object format.
}
\examples{
\dontrun{
### Using request-reply pattern.

### At the server, run next in background or the other window.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")
for(i.res in 1:5){
  buf <- zmq.recv(responder, 10L)
  cat(buf$buf, "\n")
  Sys.sleep(0.5)
  zmq.send(responder, "World")
}
zmq.close(responder)
zmq.ctx.destroy(context)


### At a client, run next in foreground.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")
for(i.req in 1:5){
  cat("Sending Hello ", i.req, "\n")
  zmq.send(requester, "Hello")
  buf <- zmq.recv(requester, 10L)
  cat("Received World ", i.req, "\n")
}
zmq.close(requester)
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_poll.r
\name{Poll Functions}
\alias{Poll Functions}
\alias{zmq.poll}
\alias{zmq.poll.free}
\alias{zmq.poll.length}
\alias{zmq.poll.get.revents}
\title{Poll Functions}
\usage{
zmq.poll(socket, type, timeout = -1L, MC = ZMQ.MC())

zmq.poll.free()

zmq.poll.length()

zmq.poll.get.revents(index = 1L)
}
\arguments{
\item{socket}{a vector of ZMQ sockets}

\item{type}{a vector of socket types corresponding to \code{socket} argument}

\item{timeout}{timeout for poll, see ZeroMQ manual for details}

\item{MC}{a message control, see \code{\link{ZMQ.MC}()} for details}

\item{index}{an index of ZMQ poll items to obtain revents}
}
\value{
\code{zmq.poll()} returns a ZMQ code and an errno,
see ZeroMQ manual for details, no error/warning/interrupt in this
\code{R} function, but some error/warning/interrupt may catch by
the \code{C} function \code{zmq_poll()}.

\code{zmq.poll.length()} returns the total number of poll items

\code{zmq.poll.get.revents()} returns the revent type
}
\description{
Poll functions
}
\details{
\code{zmq.poll()} initials ZMQ poll items given  ZMQ \code{socket}'s
and ZMQ poll \code{type}'s. Both \code{socket} and \code{type} are
in vectors of the same length, while \code{socket} contains socket pointers
and \code{type} contains types of poll.
See \code{\link{ZMQ.PO}()} for the possible values of
\code{type}. ZMQ defines several poll types and utilize
them to poll multiple sockets.

\code{zmq.poll.free()} frees ZMQ poll structure memory internally.

\code{zmq.poll.length()} obtains total numbers of ZMQ poll items.

\code{zmq.poll.get.revents()} obtains revent types from ZMQ poll item by
the input index.
}
\examples{
\dontrun{
### Using poll pattern.
### See demo/mspoller.r for details.

### Run next in background or the other window.
SHELL> Rscript wuserver.r &
SHELL> Rscript taskvent.r &
SHELL> Rscript mspoller.r

### The mspoller.r has next.
library(pbdZMQ, quietly = TRUE)

### Initial.
context <- zmq.ctx.new()
receiver <- zmq.socket(context, ZMQ.ST()$PULL)
zmq.connect(receiver, "tcp://localhost:5557")
subscriber <- zmq.socket(context, ZMQ.ST()$SUB)
zmq.connect(subscriber, "tcp://localhost:5556")
zmq.setsockopt(subscriber, ZMQ.SO()$SUBSCRIBE, "20993")

### Process messages from both sockets.
cat("Press Ctrl+C or Esc to stop mspoller.\n")
i.rec <- 0
i.sub <- 0
while(TRUE){
  ### Set poller.
  zmq.poll(c(receiver, subscriber),
           c(ZMQ.PO()$POLLIN, ZMQ.PO()$POLLIN))

  ### Check receiver.
  if(bitwAnd(zmq.poll.get.revents(1), ZMQ.PO()$POLLIN)){
    ret <- zmq.recv(receiver)
    if(ret$len != -1){
      cat("task ventilator:", ret$buf, "at", i.rec, "\n")
      i.rec <- i.rec + 1
    }
  }

  ### Check subscriber.
  if(bitwAnd(zmq.poll.get.revents(2), ZMQ.PO()$POLLIN)){
    ret <- zmq.recv(subscriber)
    if(ret$len != -1){
      cat("weather update:", ret$buf, "at", i.sub, "\n")
      i.sub <- i.sub + 1
    }
  }

  if(i.rec >= 5 & i.sub >= 5){
    break
  }

  Sys.sleep(runif(1, 0.5, 1))
}

### Finish.
zmq.poll.free()
zmq.close(receiver)
zmq.close(subscriber)
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.recv}()}, \code{\link{zmq.send}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/222_export_env.r
\docType{data}
\name{ZMQ Control Environment}
\alias{ZMQ Control Environment}
\alias{.pbd_env}
\title{Sets of controls in pbdZMQ.}
\format{
Objects contain several parameters for communicators and methods.
}
\description{
These sets of controls are used to provide default values in this package.
}
\details{
The elements of \code{.pbd_env$ZMQ.ST} are default values for socket types
as defined in `zmq.h' including \tabular{lcl}{ Elements \tab Value \tab
Usage \cr \code{PAIR} \tab 0L \tab socket type PAIR \cr \code{PUB} \tab 1L
\tab socket type PUB \cr \code{SUB} \tab 2L \tab socket type SUB \cr
\code{REQ} \tab 3L \tab socket type REQ \cr \code{REP} \tab 4L \tab socket
type REP \cr \code{DEALER} \tab 5L \tab socket type DEALER \cr \code{ROUTER}
\tab 6L \tab socket type ROUTER \cr \code{PULL} \tab 7L \tab socket type
PULL \cr \code{PUSH} \tab 8L \tab socket type PUSH \cr \code{XPUB} \tab 9L
\tab socket type XPUB \cr \code{XSUB} \tab 10L \tab socket type XSUB \cr
\code{STREAM} \tab 11L \tab socket type STREAM }

The elements of \code{.pbd_env$ZMQ.SO} are default values for socket
options as defined in `zmq.h' including 60 different values, see
\code{.pbd_env$ZMQ.SO} and `zmq.h' for details.

The elements of \code{.pbd_env$ZMQ.SR} are default values for send/recv
options as defined in `zmq.h' including \tabular{lcl}{ Elements \tab Value
\tab Usage \cr \code{BLOCK} \tab 0L \tab send/recv option BLOCK \cr
\code{DONTWAIT} \tab 1L \tab send/recv option DONTWAIT \cr \code{NOBLOCK}
\tab 1L \tab send/recv option NOBLOCK \cr \code{SNDMORE} \tab 2L \tab
send/recv option SNDMORE (not supported) }

The elements of \code{.pbd_env$ZMQ.MC} are default values for warning and
stop controls in R. These are not the ZeroMQ's internal default values. They
are defined as \tabular{lcl}{ Elements \tab Value \tab Usage \cr
\code{warning.at.error} \tab TRUE \tab if warn at error \cr
\code{stop.at.error} \tab TRUE \tab if stop at error }
}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{.zmqopt_init}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{global}
\keyword{variables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_utility.r
\name{Utility Functions}
\alias{Utility Functions}
\alias{zmq.strerror}
\alias{zmq.version}
\title{Utility Functions}
\usage{
zmq.strerror(errno)

zmq.version()
}
\arguments{
\item{errno}{an integer for the error number}
}
\value{
\code{zmq.strerror()} returns an R string containing ZeroMQ error
message.
}
\description{
Utility functions
}
\details{
\code{zmq.strerror()} gets ZeroMQ error message string.

\code{zmq.version()} print current ZeroMQ version.
}
\examples{
\dontrun{
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
zmq.ctx.destroy(context)
zmq.strerror(0)

zmq.ctx.destroy(context) # Error since context is free.
zmq.strerror(14)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.ctx.new}()}, \code{\link{zmq.ctx.destroy}()},
\code{\link{zmq.socket}()}, \code{\link{zmq.close}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random_port.r
\name{random_port}
\alias{random_port}
\alias{random_open_port}
\title{Random Port}
\usage{
random_port(min_port = 49152, max_port = 65536)

random_open_port(min_port = 49152, max_port = 65536, max_tries = 100)
}
\arguments{
\item{min_port, max_port}{The minimum/maximum value to be generated.  The minimum should not
be below 49152 and the maximum should not exceed 65536 (see
details).}

\item{max_tries}{The maximum number of times a random port will be searched for.}
}
\description{
Generate a valid, random TCP port.
}
\details{
By definition, a TCP port is an unsigned short, and so it can not
exceed 65535.  Additionally, ports in the range 1024 to 49151 are
(possibly) registered by ICANN for specific uses.

\code{random_port()} will simply generate a valid, non-registered
tcp port.  \code{random_unused_port()} will generate a port
that is available for socket connections.

\code{random_open_port()} finds a random port not already bound
to an endpoint.
}
\examples{
random_port()

}
\references{
"The Ephemeral Port Range" by Mike Gleason.  
\url{https://www.ncftp.com/ncftpd/doc/misc/ephemeral_ports.html}
}
\author{
Drew Schmidt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_socket.r
\name{Socket Functions}
\alias{Socket Functions}
\alias{zmq.socket}
\alias{zmq.close}
\alias{zmq.bind}
\alias{zmq.connect}
\alias{zmq.disconnect}
\alias{zmq.setsockopt}
\alias{zmq.getsockopt}
\title{Socket Functions}
\usage{
zmq.socket(ctx, type = ZMQ.ST()$REP)

zmq.close(socket)

zmq.bind(socket, endpoint, MC = ZMQ.MC())

zmq.connect(socket, endpoint, MC = ZMQ.MC())

zmq.disconnect(socket, endpoint, MC = ZMQ.MC())

zmq.setsockopt(socket, option.name, option.value, MC = ZMQ.MC())

zmq.getsockopt(socket, option.name, option.value, MC = ZMQ.MC())
}
\arguments{
\item{ctx}{a ZMQ context}

\item{type}{a socket type}

\item{socket}{a ZMQ socket}

\item{endpoint}{a ZMQ socket endpoint}

\item{MC}{a message control, see \code{\link{ZMQ.MC}()} for details}

\item{option.name}{an option name to the socket}

\item{option.value}{an option value to the option name}
}
\value{
\code{zmq.socket()} returns an R external pointer (\code{socket})
generated by ZMQ C API pointing to a socket if successful, otherwise returns
an R \code{NULL} and sets \code{errno} to the error value, see ZeroMQ manual
for details.

\code{zmq.close()} destroys the socket reference/pointer (\code{socket}) and
returns 0 if successful, otherwise returns -1 and sets \code{errno} to the
error value, see ZeroMQ manual for details.

\code{zmq.bind()} binds the socket to specific \code{endpoint} and returns 0
if successful, otherwise returns -1 and sets \code{errno} to the error
value, see ZeroMQ manual for details.

\code{zmq.connect()} connects the socket to specific \code{endpoint} and
returns 0 if successful, otherwise returns -1 and sets \code{errno} to the
error value, see ZeroMQ manual for details.

\code{zmq.setsockopt()} sets/changes the socket option and returns 0 if
successful, otherwise returns -1 and sets \code{errno} to the error value,
see ZeroMQ manual for details.

\code{zmq.getsockopt()} returns the value of socket option,
see ZeroMQ manual for details.
}
\description{
Socket functions
}
\details{
\code{zmq.socket()} initials a ZMQ socket given a ZMQ context \code{ctx} and
a socket \code{type}. See \code{\link{ZMQ.ST}()} for the possible values of
\code{type}. ZMQ defines several patterns for the socket type and utilize
them to communicate in different ways including request-reply,
publish-subscribe, pipeline, exclusive pair, and naive patterns.

\code{zmq.close()} destroys the ZMQ socket.

\code{zmq.bind()} binds the socket to a local endpoint and then accepts
incoming connections on that endpoint. See \code{endpoint} next for details.

\code{zmq.connect()} connects the socket to a remote endpoint and then
accepts outgoing connections on that endpoint. See \code{endpoint} next for
details.

\code{endpoint} is a string consisting of a transport :// followed by an
address. The transport specifies the underlying protocol to use. The address
specifies the transport-specific address to bind to.  pbdZMQ/ZMQ provides
the following transports: \tabular{ll}{ Transport \tab Usage \cr \code{tcp}
\tab unicast transport using TCP \cr \code{ipc} \tab local inter-process
communication transport \cr \code{inproc} \tab local in-process
(inter-thread) communication transport \cr \code{pgm,epgm} \tab reliable
multicast transport using PGM } *** warning: \code{epgm} is not turned on by
default in the pbdZMQ's internal ZeroMQ library. \cr *** warning: \code{ipc}
is not supported in Windows system.

\code{zmq.setsockopt()} is to set/change socket options.

\code{zmq.getsockopt()} is to get socket options and returns
\code{option.value}.
}
\examples{
\dontrun{
### Using request-reply pattern.

### At the server, run next in background or the other windows.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")
zmq.close(responder)
zmq.ctx.destroy(context)


### At a client, run next in foreground.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")
zmq.close(requester)
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.ctx.new}()}, \code{\link{zmq.ctx.destroy}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shellexec_wcc.r
\name{shellexec.wcc}
\alias{shellexec.wcc}
\title{Shell Execution via cmd windows}
\usage{
shellexec.wcc(file, SW.cmd = 7L)
}
\arguments{
\item{file}{a file name as in \code{shell.exec}()}

\item{SW.cmd}{a SW_* command of showing windows}
}
\value{
A new windows with certain applications depending on the association
of the input \code{file}.
}
\description{
This function is an extension to the \code{shell.exec}() which is a native
function of R
}
\details{
\code{shell.exec("a.txt")} will open a windows (notepad) to edit the file
\code{a.txt} in windows system. However, the notepad will block the (parent)
active R windows, i.e. \code{SW.cmd = 5} as \code{SH_SHOW} by default.

The \code{shellexec.wcc("a.txt", SW.cmd = 7L)} will open the notepad, but in
a minimized window. Therefore, there is no blocking to the active R windows.
See the website in the references section to see more options to control the
behavior of new windows. Possible choices are

\code{SW_SHOW (5)}: Activates the window and displays it in its current size
and position.

\code{SW_SHOWMINIMIZED (2)}: Activates the window and displays it as a
minimized window.

\code{SW_SHOWMINNOACTIVE (7)}: Displays the window as a minimized window.
The active window remains active.

\code{SW_SHOWNA (8)}: Displays the window in its current state.  The active
window remains active.
}
\examples{
\dontrun{
library(pbdZMQ, quietly = TRUE)

shellexec.wcc("a.txt", 5L)
}

}
\references{
Microsoft, Windows Dev Center: Windows desktop applications >
Develop > Desktop technologies > Desktop Environment > The Windows Shell >
Shell Reference > Shell Functions > ShellExecute

\code{https://msdn.microsoft.com/en-us/library/windows/desktop/bb762153(v=vs.85).aspx}
}
\seealso{
\code{shell.exec()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_conf_zmq.r
\name{ZMQ Flags}
\alias{ZMQ Flags}
\alias{get.zmq.ldflags}
\alias{get.zmq.cppflags}
\alias{test.load.zmq}
\alias{get.pbdZMQ.ldflags}
\title{ZMQ Flags}
\usage{
get.zmq.ldflags(arch = "", package = "pbdZMQ")

get.zmq.cppflags(arch = "", package = "pbdZMQ")

test.load.zmq(arch = "", package = "pbdZMQ")

get.pbdZMQ.ldflags(arch = "", package = "pbdZMQ")
}
\arguments{
\item{arch}{'' (default) for non-windows or '/i386' and '/ix64' for windows}

\item{package}{the pbdZMQ package}
}
\value{
flags to compile and link with ZMQ.
}
\description{
ZMQ Flags
}
\details{
\code{get.zmq.cppflags()} gets CFLAGS or CPPFLAGS

\code{get.zmq.ldflags()} gets LDFLAGS for libzmq.so, libzmq.dll, or libzmq.*.dylib

\code{get.pbdZMQ.ldflags()} gets LDFLAGS for pbdZMQ.so or pbdZMQ.dll

\code{test.load.zmq()} tests load libzmq and pbdZMQ shared libraries
}
\examples{
\dontrun{
get.zmq.cppflags(arch = '/i386')
get.zmq.ldflags(arch = '/x64')
get.pbdZMQ.ldflags(arch = '/x64')
test.load.zmq(arch = '/x64')
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{compile}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_transfers.r
\name{Transfer Functions for Files or Directories}
\alias{Transfer Functions for Files or Directories}
\alias{zmq.senddir}
\alias{zmq.recvdir}
\title{Transfer Functions for Files or Directories}
\usage{
zmq.senddir(
  port,
  infiles,
  verbose = FALSE,
  flags = ZMQ.SR()$BLOCK,
  ctx = NULL,
  socket = NULL
)

zmq.recvdir(
  port,
  endpoint,
  outfile = NULL,
  exdir = NULL,
  verbose = FALSE,
  flags = ZMQ.SR()$BLOCK,
  ctx = NULL,
  socket = NULL
)
}
\arguments{
\item{port}{A valid tcp port to be passed to \code{zmq.sendfile()} and
\code{zmq.recvfile()}.}

\item{infiles}{The name (as a string) vector of the in files to be zipped and to be
sent away.}

\item{verbose}{Logical; determines if a progress bar should be shown.}

\item{flags}{A flag for the method used by \code{zmq_sendfile} and
\code{zmq_recvfile}}

\item{ctx}{A ZMQ ctx. If \code{NULL} (default), the function will initial one at
the beginning and destroy it after finishing file transfer.}

\item{socket}{A ZMQ socket based on \code{ctx}.
If \code{NULL} (default), the function will create one at the beginning
and close it after finishing file transfer.}

\item{endpoint}{A ZMQ socket endpoint to be passed to \code{zmq.sendfile()} and
\code{zmq.recvfile()}.}

\item{outfile}{The name (as a string) of the out file to be saved on the disk.
If \code{outfile = NULL} and \code{exdir = NULL}, a tempfile will be
used and the tempfile nanme will be returned.}

\item{exdir}{The name (as a string) of the out directory to save the unzip files
unzipped from the received \code{outfile}.}
}
\value{
\code{zmq.senddir()} and \code{zmq.recvdir()} return
number of bytes (invisible) in the sent message if successful,
otherwise returns -1 (invisible) and sets \code{errno} to the error
value, see ZeroMQ manual for details.
In addition, \code{zmq.recvdir()} returns a zipped file name in a list.
}
\description{
High level functions calling \code{zmq.sendfile()} and \code{zmq.recvfile()}
to zip, transfer, and unzip small files or directories contains small files.
}
\details{
\code{zmq.senddir()} calls \code{zmq.senddir()}, and
\code{zmq.recvdir()} calls \code{zmq.recvdir()}.
}
\examples{
\dontrun{
### Run the sender and receiver code in separate R sessions.

### Receiver
library(pbdZMQ, quietly = TRUE)
zmq.recvdir(55555, "localhost", outfile = "./backup_2019.zip",
            verbose = TRUE)
### or unzip to exdir
# zmq.recvdir(55555, "localhost", exdir = "./backup_2019", verbose = TRUE)

### Sender
library(pbdZMQ, quietly = TRUE)
zmq.senddir(55555, c("./pbdZMQ/R", "./pbdZMQ/src"), verbose = TRUE)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.sendfile}()}, \code{\link{zmq.recvfile}()}.
}
\author{
Wei-Chen Chen
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_sendrecv_multipart.r
\name{Send Receive Multiple Raw Buffers}
\alias{Send Receive Multiple Raw Buffers}
\alias{zmq.send.multipart}
\alias{zmq.recv.multipart}
\title{Send Receive Multiple Raw Buffers}
\usage{
zmq.send.multipart(socket, parts, serialize = TRUE)

zmq.recv.multipart(socket, unserialize = TRUE)
}
\arguments{
\item{socket}{a ZMQ socket}

\item{parts}{a vector of multiple buffers to be sent}

\item{serialize, unserialize}{if serialize/unserialize the received multiple buffers}
}
\value{
\code{zmq.send.multipart()} returns.

\code{zmq.recv.multipart()} returns.
}
\description{
Send and receive functions for multiple raw buffers
}
\details{
\code{zmq.send.multipart()} is a high level R function to send multiple
raw messages \code{parts} at once.

\code{zmq.recv.multipart()} is a high level R function to receive multiple
raw messages at once.
}
\examples{
\dontrun{
### Using request-reply pattern.

### At the server, run next in background or the other window.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")

ret <- zmq.recv.multipart(responder, unserialize = TRUE)
parts <- as.list(rep("World", 5))
zmq.send.multipart(responder, parts)
for(i in 1:5) cat(ret[[i]])

zmq.close(responder)
zmq.ctx.destroy(context)

### At a client, run next in foreground.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")

parts <- lapply(1:5, function(i.req){ paste("Sending Hello ", i.req, "\n") })
zmq.send.multipart(requester, parts)
ret <- zmq.recv.multipart(requester, unserialize = TRUE)
print(ret)

zmq.close(requester)
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pbdZMQ-package.r
\docType{package}
\name{pbdZMQ-package}
\alias{pbdZMQ-package}
\alias{pbdZMQ}
\title{Programming with Big Data -- Interface to ZeroMQ}
\description{
ZeroMQ is a well-known library for high-performance
asynchronous messaging in scalable, distributed applications.  This
package provides high level R wrapper functions to easily utilize
ZeroMQ. We mainly focus on interactive client/server programming 
frameworks. For convenience, a minimal ZeroMQ library (4.1.0 rc1)
is shipped with pbdZMQ, which can be used if no system installation
of ZeroMQ is available.  A few wrapper functions compatible with
rzmq are also provided.
}
\details{
The install command using default \pkg{pbdZMQ}'s internal ZeroMQ library is
\cr \cr 
\code{> R CMD INSTALL pbdZMQ_0.1-0.tar.gz} \cr 
\code{--configure-args="--enable-internal-zmq"} 
\cr \cr 
Other available variables include 
\tabular{ll}{
  Variable \tab Default \cr 
  \code{ZMQ_INCLUDE} \tab \code{-I./zmqsrc/include} \cr 
  \code{ZMQ_LDFLAGS} \tab \code{-L./ -lzmq} \cr
  \code{ZMQ_POLLER} \tab \code{select} \cr
} 
See the package source file \code{pbdZMQ/configure.ac} for details.

For installation using an external ZeroMQ library, see the package source
file \code{pbdZMQ/INSTALL} for details.
}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.ctx.new}()}, \code{\link{zmq.socket}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_transfer.r
\name{File Transfer Functions}
\alias{File Transfer Functions}
\alias{zmq.sendfile}
\alias{zmq.recvfile}
\title{File Transfer Functions}
\usage{
zmq.sendfile(
  port,
  filename,
  verbose = FALSE,
  flags = ZMQ.SR()$BLOCK,
  forcebin = FALSE,
  ctx = NULL,
  socket = NULL
)

zmq.recvfile(
  port,
  endpoint,
  filename,
  verbose = FALSE,
  flags = ZMQ.SR()$BLOCK,
  forcebin = FALSE,
  ctx = NULL,
  socket = NULL
)
}
\arguments{
\item{port}{A valid tcp port.}

\item{filename}{The name (as a string) of the in/out files. The in and out file names
can be different.}

\item{verbose}{Logical; determines if a progress bar should be shown.}

\item{flags}{A flag for the method used by \code{zmq_sendfile} and
\code{zmq_recvfile}}

\item{forcebin}{Force to read/send/recv/write in binary form. Typically for a Windows
system, text (ASCII) and binary files are processed differently.
If \code{TRUE}, "r+b" and "w+b" will be enforced in the C code.
This option is mainly for Windows.}

\item{ctx}{A ZMQ ctx. If \code{NULL} (default), the function will initial one at
the beginning and destroy it after finishing file transfer.}

\item{socket}{A ZMQ socket based on \code{ctx}.
If \code{NULL} (default), the function will create one at the beginning
and close it after finishing file transfer.}

\item{endpoint}{A ZMQ socket endpoint.}
}
\value{
\code{zmq.sendfile()} and \code{zmq.recvfile()} return
number of bytes (invisible) in the sent message if successful,
otherwise returns -1 (invisible) and sets \code{errno} to the error
value, see ZeroMQ manual for details.
}
\description{
High level functions calling \code{zmq_send()} and \code{zmq_recv()}
to transfer a file in 200 KiB chunks.
}
\details{
If no socket is passed, then by default \code{zmq.sendfile()} binds a
\code{ZMQ_PUSH} socket, and \code{zmq.recvfile()} connects to this with a
\code{ZMQ_PULL} socket. On the other hand, a PUSH/PULL, REQ/REP, or REP/REQ
socket pairing may be passed. In that case, the socket should already be
connected to the desired endpoint. Be careful not to pass the wrong socket
combination (e.g., do not do REQ/REQ), as this can put the processes in an
un-recoverable state.
}
\examples{
\dontrun{
### Run the sender and receiver code in separate R sessions.

# Receiver
library(pbdZMQ, quietly = TRUE)
zmq.recvfile(55555, "localhost", "/tmp/outfile", verbose=TRUE)

# Sender
library(pbdZMQ, quietly = TRUE)
zmq.sendfile(55555, "/tmp/infile", verbose=TRUE)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.msg.send}()}, \code{\link{zmq.msg.recv}()}.
}
\author{
Drew Schmidt and Christian Heckendorf
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_message.r
\name{Message Function}
\alias{Message Function}
\alias{zmq.msg.send}
\alias{zmq.msg.recv}
\title{Message Functions}
\usage{
zmq.msg.send(
  rmsg,
  socket,
  flags = ZMQ.SR()$BLOCK,
  serialize = TRUE,
  serialversion = NULL
)

zmq.msg.recv(socket, flags = ZMQ.SR()$BLOCK, unserialize = TRUE)
}
\arguments{
\item{rmsg}{an R message}

\item{socket}{a ZMQ socket}

\item{flags}{a flag for method of send and receive}

\item{serialize}{if serialize the \code{rmsg}}

\item{serialversion}{NULL or numeric; the workspace format version to use when serializing.
NULL specifies the current default version. The only other supported
values are 2 and 3}

\item{unserialize}{if unserialize the received R message}
}
\value{
\code{zmq.msg.send()} returns 0 if successful, otherwise returns -1
and sets \code{errno} to \code{EFAULT}.

\code{zmq.msg.recv()} returns the message if successful, otherwise returns
-1 and sets \code{errno} to \code{EFAULT}.
}
\description{
Message functions
}
\details{
\code{zmq.msg.send()} sends an R message.

\code{zmq.msg.recv()} receives an R message.
}
\examples{
\dontrun{
### Using request-reply pattern.

### At the server, run next in background or the other window.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
responder <- zmq.socket(context, ZMQ.ST()$REP)
zmq.bind(responder, "tcp://*:5555")
buf <- zmq.msg.recv(responder)
set.seed(1234)
ret <- rnorm(5)
print(ret)
zmq.msg.send(ret, responder)
zmq.close(responder)
zmq.ctx.destroy(context)


### At a client, run next in foreground.
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
requester <- zmq.socket(context, ZMQ.ST()$REQ)
zmq.connect(requester, "tcp://localhost:5555")
zmq.msg.send(NULL, requester)
ret <- zmq.msg.recv(requester)
print(ret)
zmq.close(requester)
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.send}()}, \code{\link{zmq.recv}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zmq.r
\name{C-like Wrapper Functions for ZeroMQ}
\alias{C-like Wrapper Functions for ZeroMQ}
\title{The C-like ZeroMQ Interface}
\description{
The basic interface to ZeroMQ that somewhat models the C interface.
}
\details{
A list of all functions for this interface is as follows:

\tabular{lll}{
\code{zmq.bind()} \tab \code{zmq.close()} \tab \code{zmqconnect()} \cr
\code{zmq.ctx.destroy()} \tab \code{zmq.ctx.new()} \tab \code{zmq.msg.recv()} \cr
\code{zmq.msg.send()} \tab \code{zmq.recv()} \tab \code{zmq.send()} \cr
\code{zmq.socket()} \tab \tab \cr
}
}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{zmq}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/111_control.r
\name{ZMQ Control Functions}
\alias{ZMQ Control Functions}
\alias{ZMQ.MC}
\alias{ZMQ.PO}
\alias{ZMQ.SR}
\alias{ZMQ.SO}
\alias{ZMQ.ST}
\title{Sets of controls in pbdZMQ.}
\usage{
ZMQ.MC(warning.at.error = TRUE, stop.at.error = FALSE, check.eintr = FALSE)

ZMQ.PO(POLLIN = 1L, POLLOUT = 2L, POLLERR = 4L)

ZMQ.SR(BLOCK = 0L, DONTWAIT = 1L, NOBLOCK = 1L, SNDMORE = 2L)

ZMQ.SO(
  AFFINITY = 4L,
  IDENTITY = 5L,
  SUBSCRIBE = 6L,
  UNSUBSCRIBE = 7L,
  RATE = 8L,
  RECOVERY_IVL = 9L,
  SNDBUF = 11L,
  RCVBUF = 12L,
  RCVMORE = 13L,
  FD = 14L,
  EVENTS = 15L,
  TYPE = 16L,
  LINGER = 17L,
  RECONNECT_IVL = 18L,
  BACKLOG = 19L,
  RECONNECT_IVL_MAX = 21L,
  MAXMSGSIZE = 22L,
  SNDHWM = 23L,
  RCVHWM = 24L,
  MULTICAST_HOPS = 25L,
  RCVTIMEO = 27L,
  SNDTIMEO = 28L,
  LAST_ENDPOINT = 32L,
  ROUTER_MANDATORY = 33L,
  TCP_KEEPALIVE = 34L,
  TCP_KEEPALIVE_CNT = 35L,
  TCP_KEEPALIVE_IDLE = 36L,
  TCP_KEEPALIVE_INTVL = 37L,
  TCP_ACCEPT_FILTER = 38L,
  IMMEDIATE = 39L,
  XPUB_VERBOSE = 40L,
  ROUTER_RAW = 41L,
  IPV6 = 42L,
  MECHANISM = 43L,
  PLAIN_SERVER = 44L,
  PLAIN_USERNAME = 45L,
  PLAIN_PASSWORD = 46L,
  CURVE_SERVER = 47L,
  CURVE_PUBLICKEY = 48L,
  CURVE_SECRETKEY = 49L,
  CURVE_SERVERKEY = 50L,
  PROBE_ROUTER = 51L,
  REQ_CORRELATE = 52L,
  REQ_RELAXED = 53L,
  CONFLATE = 54L,
  ZAP_DOMAIN = 55L,
  ROUTER_HANDOVER = 56L,
  TOS = 57L,
  IPC_FILTER_PID = 58L,
  IPC_FILTER_UID = 59L,
  IPC_FILTER_GID = 60L,
  CONNECT_RID = 61L,
  GSSAPI_SERVER = 62L,
  GSSAPI_PRINCIPAL = 63L,
  GSSAPI_SERVICE_PRINCIPAL = 64L,
  GSSAPI_PLAINTEXT = 65L,
  HANDSHAKE_IVL = 66L,
  IDENTITY_FD = 67L,
  SOCKS_PROXY = 68L,
  XPUB_NODROP = 69L,
  BLOCKY = 70L,
  XPUB_MANUAL = 71L,
  XPUB_WELCOME_MSG = 72L,
  STREAM_NOTIFY = 73L,
  INVERT_MATCHING = 74L,
  HEARTBEAT_IVL = 75L,
  HEARTBEAT_TTL = 76L,
  HEARTBEAT_TIMEOUT = 77L,
  XPUB_VERBOSER = 78L,
  CONNECT_TIMEOUT = 79L,
  TCP_MAXRT = 80L,
  THREAD_SAFE = 81L,
  MULTICAST_MAXTPDU = 84L,
  VMCI_BUFFER_SIZE = 85L,
  VMCI_BUFFER_MIN_SIZE = 86L,
  VMCI_BUFFER_MAX_SIZE = 87L,
  VMCI_CONNECT_TIMEOUT = 88L,
  USE_FD = 89L,
  GSSAPI_PRINCIPAL_NAMETYPE = 90L,
  GSSAPI_SERVICE_PRINCIPAL_NAMETYPE = 91L,
  BINDTODEVICE = 92L,
  ZAP_ENFORCE_DOMAIN = 93L,
  LOOPBACK_FASTPATH = 94L,
  METADATA = 95L,
  MULTICAST_LOOP = 96L,
  ROUTER_NOTIFY = 97L,
  XPUB_MANUAL_LAST_VALUE = 98L,
  SOCKS_USERNAME = 99L,
  SOCKS_PASSWORD = 100L,
  IN_BATCH_SIZE = 101L,
  OUT_BATCH_SIZE = 102L,
  WSS_KEY_PEM = 103L,
  WSS_CERT_PEM = 104L,
  WSS_TRUST_PEM = 105L,
  WSS_HOSTNAME = 106L,
  WSS_TRUST_SYSTEM = 107L,
  ONLY_FIRST_SUBSCRIBE = 108L,
  RECONNECT_STOP = 109L,
  HELLO_MSG = 110L,
  DISCONNECT_MSG = 111L,
  PRIORITY = 112L
)

ZMQ.ST(
  PAIR = 0L,
  PUB = 1L,
  SUB = 2L,
  REQ = 3L,
  REP = 4L,
  DEALER = 5L,
  ROUTER = 6L,
  PULL = 7L,
  PUSH = 8L,
  XPUB = 9L,
  XSUB = 10L,
  STREAM = 11L
)
}
\arguments{
\item{warning.at.error, stop.at.error, check.eintr}{Logical; if there is a messaging error, should there be an
R warning/error, or check user interrupt events.}

\item{POLLIN, POLLOUT, POLLERR}{ZMQ poll options; see zmq.h for details.}

\item{BLOCK, DONTWAIT, NOBLOCK, SNDMORE}{ZMQ socket options; see zmq.h for details.}

\item{AFFINITY, IDENTITY, SUBSCRIBE, UNSUBSCRIBE, RATE, RECOVERY_IVL, SNDBUF, RCVBUF, RCVMORE, FD, EVENTS, TYPE, LINGER, RECONNECT_IVL, BACKLOG, RECONNECT_IVL_MAX, MAXMSGSIZE, SNDHWM, RCVHWM, MULTICAST_HOPS, RCVTIMEO, SNDTIMEO, LAST_ENDPOINT, ROUTER_MANDATORY, TCP_KEEPALIVE, TCP_KEEPALIVE_CNT, TCP_KEEPALIVE_IDLE, TCP_KEEPALIVE_INTVL, TCP_ACCEPT_FILTER, IMMEDIATE, XPUB_VERBOSE, ROUTER_RAW, IPV6, MECHANISM, PLAIN_SERVER, PLAIN_USERNAME, PLAIN_PASSWORD, CURVE_SERVER, CURVE_PUBLICKEY, CURVE_SECRETKEY, CURVE_SERVERKEY, PROBE_ROUTER, REQ_CORRELATE, REQ_RELAXED, CONFLATE, ZAP_DOMAIN, ROUTER_HANDOVER, TOS, IPC_FILTER_PID, IPC_FILTER_UID, IPC_FILTER_GID, CONNECT_RID, GSSAPI_SERVER, GSSAPI_PRINCIPAL, GSSAPI_SERVICE_PRINCIPAL, GSSAPI_PLAINTEXT, HANDSHAKE_IVL, IDENTITY_FD, SOCKS_PROXY, XPUB_NODROP, BLOCKY, XPUB_MANUAL, XPUB_WELCOME_MSG, STREAM_NOTIFY, INVERT_MATCHING, HEARTBEAT_IVL, HEARTBEAT_TTL, HEARTBEAT_TIMEOUT, XPUB_VERBOSER, CONNECT_TIMEOUT, TCP_MAXRT, THREAD_SAFE, MULTICAST_MAXTPDU, VMCI_BUFFER_SIZE, VMCI_BUFFER_MIN_SIZE, VMCI_BUFFER_MAX_SIZE, VMCI_CONNECT_TIMEOUT, USE_FD, GSSAPI_PRINCIPAL_NAMETYPE, GSSAPI_SERVICE_PRINCIPAL_NAMETYPE, BINDTODEVICE, ZAP_ENFORCE_DOMAIN, LOOPBACK_FASTPATH, METADATA, MULTICAST_LOOP, ROUTER_NOTIFY, XPUB_MANUAL_LAST_VALUE, SOCKS_USERNAME, SOCKS_PASSWORD, IN_BATCH_SIZE, OUT_BATCH_SIZE, WSS_KEY_PEM, WSS_CERT_PEM, WSS_TRUST_PEM, WSS_HOSTNAME, WSS_TRUST_SYSTEM, ONLY_FIRST_SUBSCRIBE, RECONNECT_STOP, HELLO_MSG, DISCONNECT_MSG, PRIORITY}{ZMQ socket options; see zmq.h for details.}

\item{PAIR, PUB, SUB, REQ, REP, DEALER, ROUTER, PULL, PUSH, XPUB, XSUB, STREAM}{ZMQ socket types; see zmq.h for details.}
}
\description{
These sets of controls are used to provide default values in this package.
}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{.pbd_env}}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{global}
\keyword{variables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R_zmq_context.r
\name{Context Functions}
\alias{Context Functions}
\alias{zmq.ctx.new}
\alias{zmq.ctx.destroy}
\title{Context Functions}
\usage{
zmq.ctx.new()

zmq.ctx.destroy(ctx)
}
\arguments{
\item{ctx}{a ZMQ context}
}
\value{
\code{zmq.ctx.new()} returns an R external pointer (\code{ctx})
generated by ZMQ C API pointing to a context if successful, otherwise
returns an R \code{NULL}.

\code{zmq.ctx.destroy()} returns 0 if successful, otherwise returns -1 and
sets \code{errno} to either \code{EFAULT} or \code{EINTR}.
}
\description{
Context functions
}
\details{
\code{zmq.ctx.new()} initializes a ZMQ context for starting communication.

\code{zmq.ctx.destroy()} terminates the context for stopping communication.
}
\examples{
\dontrun{
library(pbdZMQ, quietly = TRUE)

context <- zmq.ctx.new()
zmq.ctx.destroy(context)
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{zmq.socket}()}, \code{\link{zmq.close}()},
\code{\link{zmq.bind}()}, \code{\link{zmq.connect}()}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/base_ls.r
\name{ls}
\alias{ls}
\title{A wrapper function for base::ls}
\usage{
ls(
  name,
  pos = -1L,
  envir = as.environment(pos),
  all.names = FALSE,
  pattern,
  sorted = TRUE
)
}
\arguments{
\item{name, pos, envir, all.names, pattern, sorted}{as the original \code{base::ls()}.}
}
\value{
As the original \code{base::ls()} except when \code{all.names} is \code{TRUE}
and \code{envir} is \code{.GlobalEnv}, hidden pbd objects such as
\code{.pbd_env} and \code{.pbdenv} will not be returned.
}
\description{
The \code{ls()} function with modification to avoid listing hidden
pbd objects.
}
\details{
As the original \code{base::ls()}, it returns the names of the objects.
}
\examples{
\dontrun{
library(pbdRPC, quietly = TRUE)
ls(all.names = TRUE)
base::ls(all.names = TRUE)
}

}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/111_pbd_opt.r
\name{Set Control Functions}
\alias{Set Control Functions}
\alias{pbd_opt}
\title{Set controls in pbdZMQ}
\usage{
pbd_opt(..., bytext = "", envir = .GlobalEnv)
}
\arguments{
\item{...}{in argument format \code{option = value} to set
\code{.pbd_env$option <- value} inside the \code{envir}}

\item{bytext}{in text format \code{"option = value"} to set
\code{.pbd_env$option <- value} inside the \code{envir}.}

\item{envir}{by default the global environment is used.}
}
\value{
No value is returned.
}
\description{
Set control functions
}
\details{
\code{pbd_opt()} sets pbd options for ZMQ controls.

\code{...} allows multiple options in
\code{envir$.pbd_env}, but only in a simple way.

\code{bytext} allows to assign options by text in
\code{envir$.pbd_env}, but can assign advanced objects. For example,
\code{"option$suboption <- value"} will set
\code{envir$.pbd_env$option$suboption <- value}.
}
\examples{
\dontrun{
library(pbdZMQ, quietly = TRUE)

ls(.pbd_env)
rm(.pbd_env)
.zmqopt_init()
ls(.pbd_env)

.pbd_env$ZMQ.SR$BLOCK
pbd_opt(bytext = "ZMQ.SR$BLOCK <- 0L")
}

}
\references{
ZeroMQ/4.1.0 API Reference:
\url{https://libzmq.readthedocs.io/en/zeromq4-1/}

Programming with Big Data in R Website: \url{https://pbdr.org/}
}
\seealso{
\code{\link{.pbd_env}}.
}
\author{
Wei-Chen Chen \email{wccsnow@gmail.com} and Drew Schmidt.
}
\keyword{programming}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/address.r
\name{address}
\alias{address}
\title{Form an Address/Endpoint}
\usage{
address(host, port, transport = "tcp")
}
\arguments{
\item{host}{The host ip address or url.}

\item{port}{A port; necessary for all transports except ipc.}

\item{transport}{The transport protocol.  Choices are "inproc", "ipc", "tcp", and
"pgm"/"epgm" for local in-process (inter-thread), local 
inter-process, tcp, and pgm, respectively.}
}
\value{
An address, for use with pbdZMQ functions.
}
\description{
A notationally convenient function for forming addresses/endpoints.
It's a simple wrapper around the \code{paste0()} function.
}
\examples{
address("localhost", 55555)

}
\seealso{
\code{\link{zmq.bind}}
}
\author{
Drew Schmidt
}
